@@ -58,6 +58,7 @@ AtomsObject::AtomsObject(bool isLoading) : SceneObject(isLoading),
 	}
 }
 
+
 /******************************************************************************
 * Asks the object for its validity interval at the given time.
 ******************************************************************************/
@@ -580,4 +581,52 @@ QVector<FloatType> AtomsObject::getAtomRadii(TimeTicks time, TimeInterval& valid
 	return radiusArray;
 }
 
+/******************************************************************************
+* Performs a ray intersection calculation.
+******************************************************************************/
+bool AtomsObject::intersectRay(const Ray3& ray, TimeTicks time, ObjectNode* contextNode, FloatType& t, Vector3& normal)
+{
+	// Get the atomic positions and radii.
+	DataChannel* posChannel = getStandardDataChannel(DataChannel::PositionChannel);
+	if(!posChannel) return false;
+
+	TimeInterval iv;
+	QVector<FloatType> radii = getAtomRadii(time, iv);
+	OVITO_ASSERT(posChannel->size() == radii.size());
+
+	FloatType closestHitDistance = FLOATTYPE_MAX;
+
+	// Hit test every atom.
+	const Point3* p = posChannel->constDataPoint3();
+	const FloatType* r = radii.constBegin();
+	for(size_t index = 0; index < posChannel->size(); index++, ++p, ++r) {
+
+		// Perform ray-sphere intersection test.
+		Vector3 sphere_dir = *p - ray.base;
+		FloatType b = DotProduct(ray.dir, sphere_dir);
+		FloatType temp = DotProduct(sphere_dir, sphere_dir);
+		FloatType disc = b*b + square(*r) - temp;
+
+		// Only calculate the nearest intersection
+		if(disc <= 0.0)
+			continue; // Ray missed sphere entirely.
+
+		// Calculate closest intersection.
+		FloatType tnear = b - sqrt(disc);
+		if(tnear <= 0.0)
+			continue;
+
+		if(tnear < closestHitDistance) {
+			closestHitDistance = tnear;
+			normal = Normalize(ray.point(tnear) - *p);
+		}
+	}
+
+	if(closestHitDistance != FLOATTYPE_MAX) {
+		t = closestHitDistance;
+		return true;
+	}
+	else return false;
+}
+
 };	// End of namespace AtomViz
