@@ -33,8 +33,8 @@ namespace AtomViz {
 /// graphics card at a time.
 #define DEFAULT_RENDER_CHUNK_SIZE	 8000000
 
-bool AtomsRenderer::_enableHWShaders = true;
-bool AtomsRenderer::_enablePointExtension = true;
+AtomsRenderer::RenderingMethod AtomsRenderer::_defaultRenderingMethod = SHADED_POINT_SPRITE_IMPOSTERS;
+AtomsRenderer::RenderingMethod AtomsRenderer::_defaultHQRenderingMethod = SHADED_RAYTRACED_SPHERES;
 bool AtomsRenderer::_settingsLoaded = false;
 
 /******************************************************************************
@@ -46,60 +46,62 @@ void AtomsRenderer::loadRenderSettings()
 	// The OpenGL GLSL support seems to be buggy.
 	if(VIEWPORT_MANAGER.activeViewport()) {
 		const QByteArray& rendererName = VIEWPORT_MANAGER.activeViewport()->rendererName();
-		if(rendererName.contains("Mesa DRI Intel") || rendererName.contains("Software Rasterizer"))
-			_enableHWShaders = false;
+		if(rendererName.contains("Mesa DRI Intel") || rendererName.contains("Software Rasterizer")) {
+			_defaultRenderingMethod = POINT_SPRITE_IMPOSTERS;
+			_defaultHQRenderingMethod = POINT_SPRITE_IMPOSTERS;
+		}
 	}
 
 	QSettings settings;
 	settings.beginGroup("atomviz/rendering/");
-	_enableHWShaders = settings.value("UseHWShaders", qVariantFromValue(_enableHWShaders)).toBool();
-	_enablePointExtension = settings.value("UsePointExtension", qVariantFromValue(_enablePointExtension)).toBool();
+	_defaultRenderingMethod = (RenderingMethod)settings.value("DefaultRenderingMethod", qVariantFromValue((int)_defaultRenderingMethod)).toInt();
+	_defaultHQRenderingMethod = (RenderingMethod)settings.value("DefaultHQRenderingMethod", qVariantFromValue((int)_defaultHQRenderingMethod)).toInt();
 	settings.endGroup();
 
 	_settingsLoaded = true;
 }
 
 /******************************************************************************
-* Returns whether the use of the OpenGL point parameters extension is enabled for atom rendering.
+* Returns the default rendering method for atoms set by the user.
 ******************************************************************************/
-bool AtomsRenderer::arePointSpritesEnabled()
+AtomsRenderer::RenderingMethod AtomsRenderer::defaultRenderingMethod()
 {
 	if(!_settingsLoaded) loadRenderSettings();
-	return _enablePointExtension;
+	return _defaultRenderingMethod;
 }
 
 /******************************************************************************
-* Sets whether the use of the OpenGL point parameters extension should be enabled for atom rendering.
+* Returns the default high-quality rendering method for atoms set by the user.
 ******************************************************************************/
-void AtomsRenderer::enablePointSprites(bool enable)
+AtomsRenderer::RenderingMethod AtomsRenderer::defaultHQRenderingMethod()
 {
-	if(enable == _enablePointExtension) return;
-	_enablePointExtension = enable;
+	if(!_settingsLoaded) loadRenderSettings();
+	return _defaultHQRenderingMethod;
+}
+
+/******************************************************************************
+* Changes the default rendering method for atoms.
+******************************************************************************/
+void AtomsRenderer::setDefaultRenderingMethod(RenderingMethod method)
+{
+	if(method == _defaultRenderingMethod) return;
+	_defaultRenderingMethod = method;
 	QSettings settings;
 	settings.beginGroup("atomviz/rendering/");
-	settings.setValue("UsePointExtension", _enablePointExtension);
+	settings.setValue("DefaultRenderingMethod", (int)_defaultRenderingMethod);
 	settings.endGroup();
 }
 
 /******************************************************************************
-* Returns whether the use of OpenGL shader programs is enabled for atom rendering.
+* Changes the default high-quality rendering method for atoms.
 ******************************************************************************/
-bool AtomsRenderer::areHWShadersEnabled()
+void AtomsRenderer::setDefaultHQRenderingMethod(RenderingMethod method)
 {
-	if(!_settingsLoaded) loadRenderSettings();
-	return _enableHWShaders;
-}
-
-/******************************************************************************
-* Sets whether the use of OpenGL shader programs should be enabled for atom rendering.
-******************************************************************************/
-void AtomsRenderer::enableHWShaders(bool enable)
-{
-	if(enable == _enableHWShaders) return;
-	_enableHWShaders = enable;
+	if(method == _defaultHQRenderingMethod) return;
+	_defaultHQRenderingMethod = method;
 	QSettings settings;
 	settings.beginGroup("atomviz/rendering/");
-	settings.setValue("UseHWShaders", _enableHWShaders);
+	settings.setValue("DefaultHQRenderingMethod", (int)_defaultHQRenderingMethod);
 	settings.endGroup();
 }
 
@@ -107,7 +109,7 @@ void AtomsRenderer::enableHWShaders(bool enable)
 * Default constructor.
 ******************************************************************************/
 AtomsRenderer::AtomsRenderer()
-	: container(NULL), uniformRadius(0), currentAtom(NULL), hasBeenFilled(false),
+	: _window(NULL), uniformRadius(0), currentAtom(NULL), hasBeenFilled(false),
 	vboVerticesID(0), flatImposterShader(NULL), shadedImposterShader(NULL), raytracedSphereShader(NULL),
 	chunkRenderSize(DEFAULT_RENDER_CHUNK_SIZE)
 {
@@ -127,16 +129,17 @@ AtomsRenderer::~AtomsRenderer()
 ******************************************************************************/
 void AtomsRenderer::reset()
 {
-	if(container && (textureID[0] || vboVerticesID)) {
+	if(_window && (textureID[0] || vboVerticesID)) {
 		QGLContext *oldContext = const_cast<QGLContext*>(QGLContext::currentContext());
-		container->makeCurrent();
+		_window->makeCurrent();
 		// Release texture.
 		if(textureID[0])
 			CHECK_OPENGL(glDeleteTextures(2, textureID));
 		// Release vertex buffer object.
-		if(vboVerticesID)
-			CHECK_OPENGL(container->glDeleteBuffersARB(1, &vboVerticesID));
-		container = NULL;
+		if(vboVerticesID) {
+			CHECK_OPENGL(_window->glDeleteBuffersARB(1, &vboVerticesID));
+		}
+		_window = NULL;
 		if(oldContext) oldContext->makeCurrent();
 	}
 	textureID[0] = 0;
@@ -150,70 +153,87 @@ void AtomsRenderer::reset()
 
 /******************************************************************************
 * Initializes the internal buffers of this helper class if they have not been initialized yet.
-* It will be associated with the given rendering window and may subsequently
-* only be used with rendering windows that belong to the same Window3DContainer.
 ******************************************************************************/
-void AtomsRenderer::prepare(Window3D* win, bool flatShading, bool useImposters)
+AtomsRenderer::RenderingMethod AtomsRenderer::prepare(Window3D* win, bool flatAtoms, RenderingMethod renderingMethod)
 {
 	OVITO_ASSERT_MSG(APPLICATION_MANAGER.guiMode(), "AtomsRenderer::prepare()", "Rendering of atoms is only supported in GUI mode.");
 	CHECK_POINTER(win);
 
-	if(flatShading) useImposters = true;
-	if(container == NULL ||
-			(win->isSharingWith(container) == false && container != win) ||
-			useImposters != this->useImposters ||
-			flatShading != this->flatShading) {
+	if(renderingMethod == DEFAULT_RENDERING_METHOD)
+		renderingMethod = defaultRenderingMethod();
+	if(renderingMethod == DEFAULT_HQ_RENDERING_METHOD)
+		renderingMethod = defaultHQRenderingMethod();
+
+	if(flatAtoms && renderingMethod == SHADED_RAYTRACED_SPHERES)
+		renderingMethod = SHADED_POINT_SPRITE_IMPOSTERS;
+
+	if(_window == NULL ||
+			(_window != win && win->isSharingWith(_window) == false) ||
+			renderingMethod != _requestedRenderingMethod ||
+			flatAtoms != _flatAtoms) {
 		reset();
 	}
-	this->useImposters = useImposters;
-	this->flatShading = flatShading;
+	_window = win;
+	_requestedRenderingMethod = renderingMethod;
+	_flatAtoms = flatAtoms;
+	_window->makeCurrent();
 
-	container = win;
+	// Initialize OpenGL shaders.
+	if(renderingMethod == SHADED_POINT_SPRITE_IMPOSTERS || renderingMethod == SHADED_RAYTRACED_SPHERES) {
+		if(_window->hasShaderObjectsExtension() && _window->hasVertexShaderExtension() && _window->hasFragmentShaderExtension()) {
 
-	// Allocate vertex buffer object.
-	container->makeCurrent();
-	if(arePointSpritesEnabled() && !vboVerticesID && container->hasVertexBufferObjectsExtension() &&
-			container->hasPointParametersExtension() && useImposters) {
-		CHECK_OPENGL(container->glGenBuffersARB(1, &vboVerticesID));
-	}
+			// The shaders are only created once during the lifetime of the rendering window.
+			// After that they are stored along with the rendering window and will be used by all
+			// instances of the AtomsRenderer class.
 
-	// Create OpenGL shaders.
-	if(areHWShadersEnabled() && container->hasShaderObjectsExtension() &&
-			container->hasVertexShaderExtension() && container->hasFragmentShaderExtension()) {
+			try {
+				flatImposterShader = _window->findChild<OpenGLShader*>("pointsprite.flat.atom.shader");
+				if(flatImposterShader == NULL) {
+					flatImposterShader = new OpenGLShader(_window, "pointsprite.flat.atom.shader");
+					flatImposterShader->loadShader(":/atomviz/pointsprite.atom.vertexshader", ":/atomviz/flat.atom.fragmentshader");
+					OVITO_ASSERT(_window->findChild<OpenGLShader*>("pointsprite.flat.atom.shader") == flatImposterShader);
+				}
 
-		// The shaders are only created once during the lifetime of the rendering window.
-		// After that they are stored along with the rendering window and will be used by all
-		// instances of the AtomsRenderer class.
+				shadedImposterShader = _window->findChild<OpenGLShader*>("pointsprite.shaded.atom.shader");
+				if(shadedImposterShader == NULL) {
+					shadedImposterShader = new OpenGLShader(_window, "pointsprite.shaded.atom.shader");
+					shadedImposterShader->loadShader(":/atomviz/pointsprite.atom.vertexshader", ":/atomviz/shaded.atom.fragmentshader");
+					OVITO_ASSERT(_window->findChild<OpenGLShader*>("pointsprite.shaded.atom.shader") == shadedImposterShader);
+				}
 
-		try {
-			flatImposterShader = container->findChild<OpenGLShader*>("pointsprite.flat.atom.shader");
-			if(flatImposterShader == NULL) {
-				flatImposterShader = new OpenGLShader(container, "pointsprite.flat.atom.shader");
-				flatImposterShader->loadShader(":/atomviz/pointsprite.atom.vertexshader", ":/atomviz/flat.atom.fragmentshader");
-				OVITO_ASSERT(container->findChild<OpenGLShader*>("pointsprite.flat.atom.shader") == flatImposterShader);
+				raytracedSphereShader = _window->findChild<OpenGLShader*>("raytraced.atom.shader");
+				if(raytracedSphereShader == NULL) {
+					raytracedSphereShader = new OpenGLShader(_window, "raytraced.atom.shader");
+					raytracedSphereShader->loadShader(":/atomviz/raytraced.atom.vertexshader", ":/atomviz/raytraced.atom.fragmentshader");
+					OVITO_ASSERT(_window->findChild<OpenGLShader*>("raytraced.atom.shader") == raytracedSphereShader);
+				}
 			}
-
-			shadedImposterShader = container->findChild<OpenGLShader*>("pointsprite.shaded.atom.shader");
-			if(shadedImposterShader == NULL) {
-				shadedImposterShader = new OpenGLShader(container, "pointsprite.shaded.atom.shader");
-				shadedImposterShader->loadShader(":/atomviz/pointsprite.atom.vertexshader", ":/atomviz/shaded.atom.fragmentshader");
-				OVITO_ASSERT(container->findChild<OpenGLShader*>("pointsprite.shaded.atom.shader") == shadedImposterShader);
-			}
-
-			raytracedSphereShader = container->findChild<OpenGLShader*>("raytraced.atom.shader");
-			if(raytracedSphereShader == NULL) {
-				raytracedSphereShader = new OpenGLShader(container, "raytraced.atom.shader");
-				raytracedSphereShader->loadShader(":/atomviz/raytraced.atom.vertexshader", ":/atomviz/raytraced.atom.fragmentshader");
-				OVITO_ASSERT(container->findChild<OpenGLShader*>("raytraced.atom.shader") == raytracedSphereShader);
+			catch(const Exception& ex) {
+				ex.logError();
+				delete flatImposterShader;
+				delete shadedImposterShader;
+				delete raytracedSphereShader;
+				flatImposterShader = NULL;
+				shadedImposterShader = NULL;
+				raytracedSphereShader = NULL;
+				renderingMethod = POINT_SPRITE_IMPOSTERS;
 			}
 		}
-		catch(const Exception& ex) {
-			ex.logError();
-			flatImposterShader = NULL;
-			shadedImposterShader = NULL;
-			raytracedSphereShader = NULL;
-		}
+		else renderingMethod = POINT_SPRITE_IMPOSTERS;
 	}
+
+	// Allocate vertex buffer object when using point sprite extension.
+	if(renderingMethod == POINT_SPRITE_IMPOSTERS || renderingMethod == SHADED_POINT_SPRITE_IMPOSTERS) {
+		if(_window->hasVertexBufferObjectsExtension() && _window->hasPointParametersExtension()) {
+			if(vboVerticesID == 0) {
+				CHECK_OPENGL(_window->glGenBuffersARB(1, &vboVerticesID));
+			}
+		}
+		else renderingMethod = QUAD_GEOMETRY_IMPOSTERS;
+	}
+
+	_renderingMethod = renderingMethod;
+	return renderingMethod;
 }
 
 /******************************************************************************
@@ -222,9 +242,9 @@ void AtomsRenderer::prepare(Window3D* win, bool flatShading, bool useImposters)
 ******************************************************************************/
 void AtomsRenderer::beginAtoms(GLuint numAtoms)
 {
-	OVITO_ASSERT_MSG(!container.isNull(), "AtomsRenderer::BeginAtoms()", "The atoms renderer has not been initialized using the Prepare() method.");
-	if(container == NULL)
-		throw Exception("The atoms renderer has not been initialized using the Prepare() method.");
+	OVITO_ASSERT_MSG(!_window.isNull(), "AtomsRenderer::beginAtoms()", "The atoms renderer has not been initialized using the prepare() method.");
+	if(_window == NULL)
+		throw Exception("The atoms renderer has not been initialized using the prepare() method.");
 	currentAtom = NULL;
 
 	this->numAtoms = numAtoms;
@@ -234,9 +254,9 @@ void AtomsRenderer::beginAtoms(GLuint numAtoms)
 	if(vboVerticesID) {
 		// Store atoms in GPU memory.
 		internalArray.clear();
-		CHECK_OPENGL(container->glBindBufferARB(GL_ARRAY_BUFFER_ARB, vboVerticesID));
-		CHECK_OPENGL(container->glBufferDataARB(GL_ARRAY_BUFFER_ARB, numAtoms*sizeof(OpenGLAtom), NULL, GL_STREAM_DRAW_ARB));
-		CHECK_OPENGL(currentAtom = (OpenGLAtom*)container->glMapBufferARB(GL_ARRAY_BUFFER_ARB, GL_WRITE_ONLY_ARB));
+		CHECK_OPENGL(_window->glBindBufferARB(GL_ARRAY_BUFFER_ARB, vboVerticesID));
+		CHECK_OPENGL(_window->glBufferDataARB(GL_ARRAY_BUFFER_ARB, numAtoms * sizeof(OpenGLAtom), NULL, GL_STREAM_DRAW_ARB));
+		CHECK_OPENGL(currentAtom = (OpenGLAtom*)_window->glMapBufferARB(GL_ARRAY_BUFFER_ARB, GL_WRITE_ONLY_ARB));
 		CHECK_POINTER(currentAtom);
 	}
 	else {
@@ -292,8 +312,8 @@ void AtomsRenderer::endAtoms()
 
 	if(currentAtom) {
 		if(vboVerticesID) {
-			container->glUnmapBufferARB(GL_ARRAY_BUFFER_ARB);
-			container->glBindBufferARB(GL_ARRAY_BUFFER_ARB, 0);
+			_window->glUnmapBufferARB(GL_ARRAY_BUFFER_ARB);
+			_window->glBindBufferARB(GL_ARRAY_BUFFER_ARB, 0);
 		}
 		currentAtom = NULL;
 	}
@@ -307,33 +327,35 @@ void AtomsRenderer::endAtoms()
 void AtomsRenderer::render(Window3D* win)
 {
 	CHECK_POINTER(win);
-	OVITO_ASSERT_MSG(!container.isNull(), "AtomsRenderer::render()", "The atoms renderer has not been initialized using the Prepare() method.");
-	OVITO_ASSERT_MSG(container == win, "AtomsRenderer::render()", "The AtomsRenderer object may only be used with the associated Window3D.");
+	OVITO_ASSERT_MSG(!_window.isNull(), "AtomsRenderer::render()", "The atoms renderer has not been initialized using the Prepare() method.");
+	OVITO_ASSERT_MSG(_window == win, "AtomsRenderer::render()", "The AtomsRenderer object may only be used with the associated Window3D.");
 	OVITO_ASSERT_MSG(isFilled(), "AtomsRenderer::render()", "Do not call AtomsRenderer::render() before filling the render buffer with atoms.");
 
 	if(!win->isRendering())
 		return;
 
-	if(container == NULL)
+	if(_window == NULL)
 		throw Exception("The atoms renderer has not been initialized using the prepare() method.");
-	if(container != win)
+	if(_window != win)
 		throw Exception("The AtomsRenderer object may only be used with the associated Window3D.");
 
-	if(numAtoms == 0) return;
+	if(numAtoms == 0)
+		return;	// Nothing to render.
 
-	// Choose rendering method.
-	if(useImposters && container->hasPointParametersExtension() && shadedImposterShader && shadedImposterShader->isValid()) {
+	if(_renderingMethod == SHADED_POINT_SPRITE_IMPOSTERS) {
 		initializeBillboardTextures(false);
 		renderInternalImpostersWithShader(win->isPerspectiveProjection(), win->projectionMatrix(), win->viewportRectangle().height());
 	}
-	else if(!useImposters && raytracedSphereShader && raytracedSphereShader->isValid())
+	else if(_renderingMethod == SHADED_RAYTRACED_SPHERES) {
 		renderInternalRaytracedWithShaders(win->isPerspectiveProjection());
-	else {
+	}
+	else if(_renderingMethod == POINT_SPRITE_IMPOSTERS) {
 		initializeBillboardTextures();
-		if(arePointSpritesEnabled() && container->hasPointParametersExtension())
-			renderInternalArrayPointSprites(win->isPerspectiveProjection(), win->projectionMatrix(), win->viewportRectangle().height());
-		else
-			renderInternalArrayQuads();
+		renderInternalArrayPointSprites(win->isPerspectiveProjection(), win->projectionMatrix(), win->viewportRectangle().height());
+	}
+	else if(_renderingMethod == QUAD_GEOMETRY_IMPOSTERS) {
+		initializeBillboardTextures();
+		renderInternalArrayQuads();
 	}
 
 	win->enlargeSceneExtentOS(boundingBox());
@@ -345,23 +367,26 @@ void AtomsRenderer::render(Window3D* win)
 ******************************************************************************/
 void AtomsRenderer::renderOffscreen(bool isPerspective, const Matrix4& projMatrix, QSize windowSize)
 {
-	OVITO_ASSERT_MSG(!container.isNull(), "AtomsRenderer::renderOffscreen()", "The atoms renderer has not been initialized using the Prepare() method.");
+	OVITO_ASSERT_MSG(!_window.isNull(), "AtomsRenderer::renderOffscreen()", "The atoms renderer has not been initialized using the prepare() method.");
+	OVITO_ASSERT_MSG(isFilled(), "AtomsRenderer::renderOffscreen()", "Do not call AtomsRenderer::render() before filling the render buffer with atoms.");
 
-	if(numAtoms == 0) return;
+	if(numAtoms == 0)
+		return;	// Nothing to render.
 
-	// Choose rendering method.
-	if(useImposters && container->hasPointParametersExtension() && shadedImposterShader && shadedImposterShader->isValid()) {
+	if(_renderingMethod == SHADED_POINT_SPRITE_IMPOSTERS) {
 		initializeBillboardTextures(false);
 		renderInternalImpostersWithShader(isPerspective, projMatrix, windowSize.height());
 	}
-	else if(!useImposters && raytracedSphereShader && raytracedSphereShader->isValid())
+	else if(_renderingMethod == SHADED_RAYTRACED_SPHERES) {
 		renderInternalRaytracedWithShaders(isPerspective);
-	else {
+	}
+	else if(_renderingMethod == POINT_SPRITE_IMPOSTERS) {
 		initializeBillboardTextures();
-		if(container->hasPointParametersExtension())
-			renderInternalArrayPointSprites(isPerspective, projMatrix, windowSize.height());
-		else
-			renderInternalArrayQuads();
+		renderInternalArrayPointSprites(isPerspective, projMatrix, windowSize.height());
+	}
+	else if(_renderingMethod == QUAD_GEOMETRY_IMPOSTERS) {
+		initializeBillboardTextures();
+		renderInternalArrayQuads();
 	}
 }
 
@@ -371,8 +396,10 @@ void AtomsRenderer::renderOffscreen(bool isPerspective, const Matrix4& projMatri
 void AtomsRenderer::initializeBillboardTextures(bool withAlpha)
 {
 	glEnable(GL_TEXTURE_2D);
-	glAlphaFunc(GL_GREATER, 0);
-	glEnable(GL_ALPHA_TEST);
+	//if(withAlpha) {
+		glAlphaFunc(GL_GREATER, 0);
+		glEnable(GL_ALPHA_TEST);
+	//}
 
 	// Generate texture image.
 	static GLubyte shadedImage[BILLBOARD_TEXTURE_SIZE][BILLBOARD_TEXTURE_SIZE][4];
@@ -386,8 +413,8 @@ void AtomsRenderer::initializeBillboardTextures(bool withAlpha)
 			for(int dy=-BILLBOARD_TEXTURE_SIZE/2; dy<BILLBOARD_TEXTURE_SIZE/2; dy++, shadedPixel += 4, flatPixel += 4) {
 				int dist = dx*dx + dy*dy;
 				int diff = BILLBOARD_TEXTURE_SIZE*BILLBOARD_TEXTURE_SIZE/4 - dist;
-				if(diff >= 0 || !withAlpha) {
 
+				if(diff >= 0) {
 					Vector3 normal(dx, dy, 0);
 
 					normal.Z = sqrt((FloatType)max(diff, 0));
@@ -399,14 +426,11 @@ void AtomsRenderer::initializeBillboardTextures(bool withAlpha)
 					FloatType specular = pow(max((FloatType)0.0, -reflect.Z), shininess) * 0.25;
 					FloatType ambient = 0.1;
 					FloatType colorValue = (diffuse + ambient);
-					if(withAlpha) colorValue += specular;
+					//if(withAlpha) colorValue += specular;
 
 					GLubyte color = (GLubyte)(min(colorValue, (FloatType)1.0) * 255.0);
 					shadedPixel[0] = shadedPixel[1] = shadedPixel[2] = color;
-					if(!withAlpha)
-						shadedPixel[3] = GLubyte(min(specular, (FloatType)1.0) * 255.0);
-					else
-						shadedPixel[3] = 255;
+					shadedPixel[3] = 255;
 					flatPixel[0] = flatPixel[1] = flatPixel[2] = flatPixel[3] = 255;
 				}
 				else {
@@ -423,18 +447,16 @@ void AtomsRenderer::initializeBillboardTextures(bool withAlpha)
 
 		CHECK_OPENGL(glBindTexture(GL_TEXTURE_2D, textureID[0]));
 		CHECK_OPENGL(glTexImage2D(GL_TEXTURE_2D, 0, 4, BILLBOARD_TEXTURE_SIZE, BILLBOARD_TEXTURE_SIZE, 0, GL_RGBA, GL_UNSIGNED_BYTE, shadedImage));
-	//	CHECK_OPENGL(glTexImage2D(GL_TEXTURE_2D, 0, 4, BILLBOARD_TEXTURE_SIZE, BILLBOARD_TEXTURE_SIZE, 0, GL_RGBA, GL_FLOAT, shadedImage));
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 
 		CHECK_OPENGL(glBindTexture(GL_TEXTURE_2D, textureID[1]));
 		CHECK_OPENGL(glTexImage2D(GL_TEXTURE_2D, 0, 4, BILLBOARD_TEXTURE_SIZE, BILLBOARD_TEXTURE_SIZE, 0, GL_RGBA, GL_UNSIGNED_BYTE, flatImage));
-	//	CHECK_OPENGL(glTexImage2D(GL_TEXTURE_2D, 0, 4, BILLBOARD_TEXTURE_SIZE, BILLBOARD_TEXTURE_SIZE, 0, GL_RGBA, GL_FLOAT, flatImage));
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 	}
 
-	glBindTexture(GL_TEXTURE_2D, textureID[flatShading ? 1 : 0]);
+	glBindTexture(GL_TEXTURE_2D, textureID[_flatAtoms ? 1 : 0]);
 }
 
 /******************************************************************************
@@ -444,6 +466,7 @@ void AtomsRenderer::initializeBillboardTextures(bool withAlpha)
 void AtomsRenderer::renderInternalArrayPointSprites(bool isPerspective, const Matrix4& projMatrix, float windowHeight)
 {
 	OVITO_ASSERT_MSG(windowHeight > 0, "AtomsRenderer::renderInternalArrayPointSprites()", "The viewport window has zero height.");
+	OVITO_ASSERT(_window->hasPointParametersExtension());
 
 	// Disable lighting.
 	glPushAttrib(GL_LIGHTING_BIT);
@@ -457,19 +480,19 @@ void AtomsRenderer::renderInternalArrayPointSprites(bool isPerspective, const Ma
 	// distance from the viewer
 	if(isPerspective) {
 		float quadratic[] = {0.0f, 0.0f, 100.0f / (projMatrix(1,1)*projMatrix(1,1) * windowHeight * windowHeight) };
-		container->glPointParameterfvARB(GL_POINT_DISTANCE_ATTENUATION_ARB, quadratic);
+		_window->glPointParameterfvARB(GL_POINT_DISTANCE_ATTENUATION_ARB, quadratic);
 		if(uniformRadius > 0)
 			CHECK_OPENGL(glPointSize(uniformRadius * 10.0));
 	}
 	else {
 		float constant[] = {1.0f, 0.0f, 0.0f};
-		container->glPointParameterfvARB(GL_POINT_DISTANCE_ATTENUATION_ARB, constant);
+		_window->glPointParameterfvARB(GL_POINT_DISTANCE_ATTENUATION_ARB, constant);
 		if(uniformRadius > 0)
 			CHECK_OPENGL(glPointSize(uniformRadius * projMatrix(1,1) * windowHeight));
 	}
 	// No fading of small points.
-	CHECK_OPENGL(container->glPointParameterfARB(GL_POINT_FADE_THRESHOLD_SIZE_ARB, 0.0f));
-	CHECK_OPENGL(container->glPointParameterfARB(GL_POINT_SIZE_MIN_ARB, 0.01f));
+	CHECK_OPENGL(_window->glPointParameterfARB(GL_POINT_FADE_THRESHOLD_SIZE_ARB, 0.0f));
+	CHECK_OPENGL(_window->glPointParameterfARB(GL_POINT_SIZE_MIN_ARB, 0.01f));
 
 	// Specify point sprite texture coordinate replacement mode for each texture unit.
 	glTexEnvf(GL_POINT_SPRITE_ARB, GL_COORD_REPLACE_ARB, GL_TRUE);
@@ -478,7 +501,7 @@ void AtomsRenderer::renderInternalArrayPointSprites(bool isPerspective, const Ma
 	CHECK_OPENGL(glEnableClientState(GL_VERTEX_ARRAY));
 	CHECK_OPENGL(glEnableClientState(GL_COLOR_ARRAY));
 	if(vboVerticesID) {
-		CHECK_OPENGL(container->glBindBufferARB(GL_ARRAY_BUFFER_ARB, vboVerticesID));
+		CHECK_OPENGL(_window->glBindBufferARB(GL_ARRAY_BUFFER_ARB, vboVerticesID));
 		CHECK_OPENGL(glVertexPointer(3, GL_FLOAT, sizeof(OpenGLAtom), (const char*)NULL +  offsetof(OpenGLAtom, x)));
 		CHECK_OPENGL(glColorPointer(3, GL_UNSIGNED_BYTE, sizeof(OpenGLAtom), (const char*)NULL + offsetof(OpenGLAtom, r)));
 	}
@@ -486,8 +509,8 @@ void AtomsRenderer::renderInternalArrayPointSprites(bool isPerspective, const Ma
 		CHECK_OPENGL(glVertexPointer(3, GL_FLOAT, sizeof(OpenGLAtom), &internalArray.data()->x));
 		CHECK_OPENGL(glColorPointer(3, GL_UNSIGNED_BYTE, sizeof(OpenGLAtom), &internalArray.data()->r));
 	}
-	if(container->hasCompiledVertexArraysExtension())
-		CHECK_OPENGL(container->glLockArrays(0, (GLsizei)numAtoms));
+	if(_window->hasCompiledVertexArraysExtension())
+		CHECK_OPENGL(_window->glLockArrays(0, (GLsizei)numAtoms));
 
 	if(chunkRenderSize == 0) {
 		// Render atoms all at once.
@@ -500,12 +523,11 @@ void AtomsRenderer::renderInternalArrayPointSprites(bool isPerspective, const Ma
 		}
 	}
 
-	if(container->hasCompiledVertexArraysExtension())
-		container->glUnlockArrays();
+	if(_window->hasCompiledVertexArraysExtension())
+		_window->glUnlockArrays();
 
-	if(vboVerticesID) {
-		container->glBindBufferARB(GL_ARRAY_BUFFER_ARB, 0);
-	}
+	if(vboVerticesID)
+		_window->glBindBufferARB(GL_ARRAY_BUFFER_ARB, 0);
 
 	glDisableClientState(GL_COLOR_ARRAY);
 	glDisableClientState(GL_VERTEX_ARRAY);
@@ -525,13 +547,15 @@ void AtomsRenderer::renderInternalImpostersWithShader(bool isPerspective, const
 {
 	OVITO_ASSERT(shadedImposterShader && shadedImposterShader->isValid());
 	OVITO_ASSERT(flatImposterShader && flatImposterShader->isValid());
+	OVITO_ASSERT(_window->hasPointParametersExtension());
 	OVITO_ASSERT_MSG(windowHeight > 0, "AtomsRenderer::renderInternalImpostersWithShader()", "The viewport window has zero height.");
 
 	// Disable lighting.
 	glPushAttrib(GL_LIGHTING_BIT);
 	glDisable(GL_LIGHTING);
 	glDisable(GL_BLEND);
-	glDisable(GL_ALPHA_TEST);
+	//glDisable(GL_ALPHA_TEST);
+	glEnable(GL_TEXTURE_2D);
 
 	// Use point sprites.
 	glEnable(GL_POINT_SPRITE_ARB);
@@ -540,24 +564,24 @@ void AtomsRenderer::renderInternalImpostersWithShader(bool isPerspective, const
 	// distance from the viewer
 	if(isPerspective) {
 		float quadratic[] = {0.0f, 0.0f, 100.0f / (projMatrix(1,1)*projMatrix(1,1) * windowHeight * windowHeight) };
-		CHECK_OPENGL(container->glPointParameterfvARB(GL_POINT_DISTANCE_ATTENUATION_ARB, quadratic));
+		CHECK_OPENGL(_window->glPointParameterfvARB(GL_POINT_DISTANCE_ATTENUATION_ARB, quadratic));
 		if(uniformRadius > 0)
 			CHECK_OPENGL(glPointSize(uniformRadius * 10.0));
 	}
 	else {
 		float constant[] = {1.0f, 0.0f, 0.0f};
-		CHECK_OPENGL(container->glPointParameterfvARB(GL_POINT_DISTANCE_ATTENUATION_ARB, constant));
+		CHECK_OPENGL(_window->glPointParameterfvARB(GL_POINT_DISTANCE_ATTENUATION_ARB, constant));
 		if(uniformRadius > 0)
 			CHECK_OPENGL(glPointSize(uniformRadius * projMatrix(1,1) * windowHeight));
 	}
 	// No fading of small points.
-	CHECK_OPENGL(container->glPointParameterfARB(GL_POINT_FADE_THRESHOLD_SIZE_ARB, 0.0f));
-	CHECK_OPENGL(container->glPointParameterfARB(GL_POINT_SIZE_MIN_ARB, 0.01f));
+	CHECK_OPENGL(_window->glPointParameterfARB(GL_POINT_FADE_THRESHOLD_SIZE_ARB, 0.0f));
+	CHECK_OPENGL(_window->glPointParameterfARB(GL_POINT_SIZE_MIN_ARB, 0.01f));
 
 	// Specify point sprite texture coordinate replacement mode for each texture unit.
 	glTexEnvf(GL_POINT_SPRITE_ARB, GL_COORD_REPLACE_ARB, GL_TRUE);
 
-	OpenGLShader* shader = flatShading ? flatImposterShader : shadedImposterShader;
+	OpenGLShader* shader = _flatAtoms ? flatImposterShader : shadedImposterShader;
 	shader->setEnabled(true);
 	// Let the vertex shader compute the point size.
 	glEnable(GL_VERTEX_PROGRAM_POINT_SIZE);
@@ -568,25 +592,26 @@ void AtomsRenderer::renderInternalImpostersWithShader(bool isPerspective, const
 	glEnableClientState(GL_VERTEX_ARRAY);
 	glEnableClientState(GL_COLOR_ARRAY);
 	if(vboVerticesID) {
-		CHECK_OPENGL(container->glBindBufferARB(GL_ARRAY_BUFFER_ARB, vboVerticesID));
+		CHECK_OPENGL(_window->glBindBufferARB(GL_ARRAY_BUFFER_ARB, vboVerticesID));
 		glVertexPointer(3, GL_FLOAT, sizeof(OpenGLAtom), (const char*)NULL + offsetof(OpenGLAtom, x));
 		glColorPointer(3, GL_UNSIGNED_BYTE, sizeof(OpenGLAtom), (const char*)NULL + offsetof(OpenGLAtom, r));
-		// The fog coord array is abused to pass the atom radii to the shader.
-		if(container->hasFogCoordExtension() && shader) {
+		// The fog coord array is used to pass the atom radii to the shader.
+		if(_window->hasFogCoordExtension() && shader) {
 			glEnableClientState(GL_FOG_COORD_ARRAY);
-			container->glFogCoordPointerEXT(GL_FLOAT, sizeof(OpenGLAtom), (const char*)NULL + offsetof(OpenGLAtom, radius));
+			_window->glFogCoordPointerEXT(GL_FLOAT, sizeof(OpenGLAtom), (const char*)NULL + offsetof(OpenGLAtom, radius));
 		}
 	}
 	else {
 		glVertexPointer(3, GL_FLOAT, sizeof(OpenGLAtom), &internalArray.data()->x);
 		glColorPointer(3, GL_UNSIGNED_BYTE, sizeof(OpenGLAtom), &internalArray.data()->r);
-		// The fog coord array is abused to pass the atom radii to the shader.
-		if(container->hasFogCoordExtension() && shader) {
+		// The fog coord array is used to pass the atom radii to the shader.
+		if(_window->hasFogCoordExtension() && shader) {
 			glEnableClientState(GL_FOG_COORD_ARRAY);
-			container->glFogCoordPointerEXT(GL_FLOAT, sizeof(OpenGLAtom), &internalArray.data()->radius);
+			_window->glFogCoordPointerEXT(GL_FLOAT, sizeof(OpenGLAtom), &internalArray.data()->radius);
 		}
 	}
-	if(container->hasCompiledVertexArraysExtension()) CHECK_OPENGL(container->glLockArrays(0, (GLsizei)numAtoms));
+	if(_window->hasCompiledVertexArraysExtension())
+		CHECK_OPENGL(_window->glLockArrays(0, (GLsizei)numAtoms));
 
 	if(chunkRenderSize == 0) {
 		// Render atoms all at once.
@@ -599,18 +624,18 @@ void AtomsRenderer::renderInternalImpostersWithShader(bool isPerspective, const
 		}
 	}
 
-	if(container->hasCompiledVertexArraysExtension()) container->glUnlockArrays();
+	if(_window->hasCompiledVertexArraysExtension())
+		_window->glUnlockArrays();
 
 	shader->setEnabled(false);
 	glDisable(GL_VERTEX_PROGRAM_POINT_SIZE);
 
-	if(vboVerticesID) {
-		container->glBindBufferARB(GL_ARRAY_BUFFER_ARB, 0);
-	}
+	if(vboVerticesID)
+		_window->glBindBufferARB(GL_ARRAY_BUFFER_ARB, 0);
 
 	glDisableClientState(GL_COLOR_ARRAY);
 	glDisableClientState(GL_VERTEX_ARRAY);
-	if(container->hasFogCoordExtension() && shader)
+	if(_window->hasFogCoordExtension() && shader)
 		glDisableClientState(GL_FOG_COORD_ARRAY);
 
 	// Cleanup
@@ -632,6 +657,7 @@ void AtomsRenderer::renderInternalRaytracedWithShaders(bool isPerspective)
 	glDisable(GL_LIGHTING);
 	glDisable(GL_BLEND);
 	glDisable(GL_ALPHA_TEST);
+	glDisable(GL_TEXTURE_2D);
 
 	raytracedSphereShader->setEnabled(true);
 	raytracedSphereShader->sendUniform1i("isPerspective", isPerspective);
@@ -658,8 +684,8 @@ void AtomsRenderer::renderInternalRaytracedWithShaders(bool isPerspective)
 		float zmax = atom->z + atom->radius;
 
 		glColor4ubv(&atom->r);
-		container->glVertexAttrib1fARB(particleRadiusLocation, atom->radius);
-		container->glVertexAttrib3fvARB(particlePosLocation, &atom->x);
+		_window->glVertexAttrib1fARB(particleRadiusLocation, atom->radius);
+		_window->glVertexAttrib3fvARB(particlePosLocation, &atom->x);
 
 		// -Z face
 		glVertex3f(xmin, ymax, zmin);
@@ -734,13 +760,13 @@ void AtomsRenderer::renderInternalArrayQuads()
 		float z = m[0][2]*atom->x + m[1][2]*atom->y + m[2][2]*atom->z + m[3][2];
 
 		glColor4ubv(&atom->r);
-		glTexCoord2f(0, 0);
-		glVertex3f(x - atom->radius, y - atom->radius, z);
-		glTexCoord2f(1, 0);
-		glVertex3f(x + atom->radius, y - atom->radius, z);
-		glTexCoord2f(1, 1);
-		glVertex3f(x + atom->radius, y + atom->radius, z);
 		glTexCoord2f(0, 1);
+		glVertex3f(x - atom->radius, y - atom->radius, z);
+		glTexCoord2f(1, 1);
+		glVertex3f(x + atom->radius, y - atom->radius, z);
+		glTexCoord2f(1, 0);
+		glVertex3f(x + atom->radius, y + atom->radius, z);
+		glTexCoord2f(0, 0);
 		glVertex3f(x - atom->radius, y + atom->radius, z);
 	}
 	glEnd();
@@ -854,8 +880,8 @@ bool AtomsRenderer::calculateAmbientOcclusion(Window3D* glcontext)
 	glLoadMatrix(projMatrix.constData());
 	glMatrixMode(GL_MODELVIEW);
 
-	bool oldFlatShading = flatShading;
-	flatShading = true;
+	bool oldFlatAtoms = _flatAtoms;
+	_flatAtoms = true;
 	QVector<float> depthBuffer(fbWidth*fbHeight);
 
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
@@ -863,11 +889,11 @@ bool AtomsRenderer::calculateAmbientOcclusion(Window3D* glcontext)
 	glTranslatef(-(GLfloat)center.X, -(GLfloat)center.Y, -(GLfloat)center.Z);
 
 	// Choose rendering method.
-	if(useImposters && container->hasPointParametersExtension() && flatImposterShader && flatImposterShader->isValid())
+	if(_window->hasPointParametersExtension() && flatImposterShader && flatImposterShader->isValid())
 		renderInternalImpostersWithShader(false, projMatrix, fbHeight);
 	else {
 		initializeBillboardTextures();
-		if(container->hasPointParametersExtension())
+		if(_window->hasPointParametersExtension())
 			renderInternalArrayPointSprites(false, projMatrix, fbHeight);
 		else
 			renderInternalArrayQuads();
@@ -889,7 +915,7 @@ bool AtomsRenderer::calculateAmbientOcclusion(Window3D* glcontext)
 	*/
 
 	// Restore old settings.
-	flatShading = oldFlatShading;
+	_flatAtoms = oldFlatAtoms;
 	glMatrixMode(GL_PROJECTION);
 	glPopMatrix();
 	glMatrixMode(GL_MODELVIEW);
