@@ -27,6 +27,10 @@
 #include <core/data/units/ParameterUnit.h>
 #include <core/viewport/ViewportManager.h>
 
+#if !defined(Q_WS_MAC)
+	#include <GL/glu.h>
+#endif
+
 #include "SimulationCell.h"
 #include "AtomsObject.h"
 
@@ -317,7 +321,9 @@ void SimulationCellEditor::createUI(const RolloutInsertionParameters& rolloutPar
 
 		QGridLayout* layout2 = new QGridLayout(sizeGroupBox);
 		layout2->setContentsMargins(4,4,4,4);
+#ifndef Q_WS_MAC
 		layout2->setSpacing(0);
+#endif
 		layout2->setColumnStretch(1, 1);
 
 		QSignalMapper* signalMapperValueChanged = new QSignalMapper(this);
@@ -359,20 +365,21 @@ void SimulationCellEditor::createUI(const RolloutInsertionParameters& rolloutPar
 
 		QGridLayout* layout2 = new QGridLayout(renderingGroupBox);
 		layout2->setContentsMargins(4,4,4,4);
+#ifndef Q_WS_MAC
 		layout2->setSpacing(0);
+#endif
 		layout2->setColumnStretch(1, 1);
 
 		// Simulation cell line width parameter.
 		lineWidthUI = new FloatPropertyUI(this, PROPERTY_FIELD_DESCRIPTOR(SimulationCell, _simulationCellLineWidth));
 		layout2->addWidget(lineWidthUI->label(), 0, 0);
-		layout2->addWidget(lineWidthUI->textBox(), 0, 1);
-		layout2->addWidget(lineWidthUI->spinner(), 0, 2);
+		layout2->addLayout(lineWidthUI->createFieldLayout(), 0, 1);
 		lineWidthUI->setMinValue(0);
 
 		// Background color parameter.
 		ColorPropertyUI* cellColorPUI = new ColorPropertyUI(this, PROPERTY_FIELD_DESCRIPTOR(SimulationCell, _simulationCellColor));
 		layout2->addWidget(cellColorPUI->label(), 1, 0);
-		layout2->addWidget(cellColorPUI->colorPicker(), 1, 1, 1, 2);
+		layout2->addWidget(cellColorPUI->colorPicker(), 1, 1);
 	}
 	layout1->addStretch(1);
 
