@@ -272,21 +272,12 @@ EvaluationStatus AmbientLightingModifier::doAnalysis(TimeTicks time, bool suppre
 	// Release old data
 	ambientArray.clear();
 
-	bool useOnscreenRendering = false;
-
-	// ATI Mobility Radeon X1400 driver's support of the frame buffer objects extension is buggy.
-	// Have to do on-screen rendering on this platform.
-	if(QByteArray((const char*)glGetString(GL_RENDERER)).contains("ATI Mobility Radeon X1400"))
-		useOnscreenRendering = true;
-
 	// Check for the required OpenGL extensions to render the atoms.
 	if(APPLICATION_MANAGER.guiMode() == false || VIEWPORT_MANAGER.activeViewport() == NULL)
 		throw Exception(tr("Cannot perform ambient lighting calculation in non-interactive mode."));
 	const OpenGLExtensions& ogl = *(VIEWPORT_MANAGER.activeViewport());
-	if(!ogl.hasFrameBufferExtension() && !QGLPixelBuffer::hasOpenGLPbuffers()) {
-		MsgLogger() << tr("Warning: The OpenGL driver does not support the EXT_framebuffer_object or the ARB_pixel_buffer_object extension. Falling back to on-screen rendering for ambient lighting.") << endl;
-		useOnscreenRendering = true;
-	}
+	if(!QGLPixelBuffer::hasOpenGLPbuffers())
+		throw Exception(tr("The OpenGL driver does not support the EXT_framebuffer_object or the ARB_pixel_buffer_object extension."));
 
 	// Get the atom positions.
 	DataChannel* posChannel = expectStandardChannel(DataChannel::PositionChannel);
@@ -320,31 +311,13 @@ EvaluationStatus AmbientLightingModifier::doAnalysis(TimeTicks time, bool suppre
 	default: samplingPoints = sphericalSampling1; numSamplingPoints = sizeof(sphericalSampling1)/sizeof(sphericalSampling1[0]); break;
 	}
 
-	ProgressIndicator progress(tr("Computing ambient lighting."), numSamplingPoints, useOnscreenRendering || suppressDialogs);
+	ProgressIndicator progress(tr("Computing ambient lighting."), numSamplingPoints, suppressDialogs);
 
-	Window3D* renderingWindow = NULL;
-	Window3DContainer* renderingContainer;
-	if(useOnscreenRendering) {
-		// Create the rendering window for on-screen rendering.
-
-		class OnscreenRenderingWindow : public Window3D {
-		public:
-			OnscreenRenderingWindow(Window3DContainer* container) : Window3D(container) {}
-		protected:
-			virtual void renderWindow() {}
-		};
-
-		renderingContainer = new Window3DContainer();
-		renderingContainer->resize(renderBufferSize, renderBufferSize);
-		renderingContainer->move(0,0);
-		renderingWindow = new OnscreenRenderingWindow(renderingContainer);
-		renderingWindow->setGeometry(renderingContainer->rect());
-		renderingContainer->show();
-	}
-	else renderingWindow = VIEWPORT_MANAGER.activeViewport();
+	Window3D* renderingWindow = VIEWPORT_MANAGER.activeViewport();
+	renderingWindow->makeCurrent();		
 
 	{
-
+	
 	// Prepare atom renderer
 	AtomsRenderer atomRenderer;
 	atomRenderer.prepare(renderingWindow, true);
@@ -361,55 +334,13 @@ EvaluationStatus AmbientLightingModifier::doAnalysis(TimeTicks time, bool suppre
 	// Allocate ambient array.
 	ambientArray.fill(0, numAtoms);
 
-	GLuint frameBuffer;
-	GLuint renderBuffer;
-	GLuint depthRenderBuffer;
+	// Use OpenGL pixel buffer for off-screen rendering.	
+	VerboseLogger() << logdate << "Allocating OpenGL off-screen pixel buffer:" << renderBufferSize << "x" << renderBufferSize << "pixels" << endl;
 	scoped_ptr<QGLPixelBuffer> pbuffer;
+	pbuffer.reset(new QGLPixelBuffer(renderBufferSize, renderBufferSize, QGLFormat(QGL::DepthBuffer | QGL::Rgba | QGL::DirectRendering | QGL::NoAccumBuffer | QGL::NoStencilBuffer), renderingWindow));
 
-	if(useOnscreenRendering) {
-		renderingWindow->makeCurrent();
-	}
-	else if(renderingWindow->hasFrameBufferExtension()) {
-		// Use OpenGL frame buffer for off-screen rendering.
-
-	    // Create the framebuffer object - make sure to have a current
-	    // context before creating it.
-		renderingWindow->makeCurrent();
-
-		// Create a frame-buffer object and a render-buffer object for color and depth channel.
-		VerboseLogger() << logdate << "Allocating OpenGL off-screen frame buffer:" << renderBufferSize << "x" << renderBufferSize << "pixels" << endl;
-		CHECK_OPENGL(renderingWindow->glGenFramebuffersEXT(1, &frameBuffer));
-		CHECK_OPENGL(renderingWindow->glGenRenderbuffersEXT(1, &renderBuffer));
-		CHECK_OPENGL(renderingWindow->glGenRenderbuffersEXT(1, &depthRenderBuffer));
-
-		// Initialize the render-buffers.
-		VerboseLogger() << logdate << "Binding OpenGL framebuffer." << endl;
-		CHECK_OPENGL(renderingWindow->glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, depthRenderBuffer));
-		CHECK_OPENGL(renderingWindow->glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, GL_DEPTH_COMPONENT24, renderBufferSize, renderBufferSize));
-		CHECK_OPENGL(renderingWindow->glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, renderBuffer));
-		CHECK_OPENGL(renderingWindow->glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, GL_RGBA8, renderBufferSize, renderBufferSize));
-
-		// Activate the off-screen frame-buffer and associate the render-buffer objects.
-		CHECK_OPENGL(renderingWindow->glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, frameBuffer));
-		CHECK_OPENGL(renderingWindow->glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_RENDERBUFFER_EXT, renderBuffer));
-		CHECK_OPENGL(renderingWindow->glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, depthRenderBuffer));
-
-		// Check for errors...
-		GLenum status = renderingWindow->glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
-		if(status != GL_FRAMEBUFFER_COMPLETE_EXT)
-			throw Exception(tr("The OpenGL framebuffer object is not valid."));
-
-		CHECK_OPENGL(glReadBuffer(GL_COLOR_ATTACHMENT0_EXT));
-	}
-	else {
-		// Use OpenGL pixel buffer for off-screen rendering.
-
-		VerboseLogger() << logdate << "Allocating OpenGL off-screen pixel buffer:" << renderBufferSize << "x" << renderBufferSize << "pixels" << endl;
-		pbuffer.reset(new QGLPixelBuffer(renderBufferSize, renderBufferSize, QGLFormat(QGL::DepthBuffer | QGL::Rgba | QGL::DirectRendering | QGL::NoAccumBuffer | QGL::NoStencilBuffer)));
-
-		if(!pbuffer->makeCurrent())
-			throw Exception(tr("The OpenGL pixel buffer could not be activated."));
-	}
+	if(!pbuffer->makeCurrent())
+		throw Exception(tr("The OpenGL pixel buffer could not be activated."));
 
 	// Prepare OpenGL rendering state.
 	CHECK_OPENGL(glPushAttrib(GL_LIGHTING_BIT|GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT|GL_ENABLE_BIT|GL_VIEWPORT_BIT|GL_TRANSFORM_BIT));
@@ -435,36 +366,45 @@ EvaluationStatus AmbientLightingModifier::doAnalysis(TimeTicks time, bool suppre
 	CHECK_OPENGL(glLoadMatrix(projMatrix.constData()));
 	CHECK_OPENGL(glMatrixMode(GL_MODELVIEW));
 
-	// Allocate user-memory pixel buffer.
-	scoped_array<quint32> pixelData(new quint32[renderBufferSize * renderBufferSize]);
-
-	for(size_t samplingIndex=0; samplingIndex<numSamplingPoints; samplingIndex++) {
-		if(progress.isCanceled()) break;
-		progress.setValue(samplingIndex);
+	if(!progress.isCanceled()) {
+		progress.setValue(0);
 
+		// Do a first rendering pass, but don't use the results.
+		// On the Windows platform, the first rendered off-screen image is garbage.
 		CHECK_OPENGL(glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT));
-
-		glLoadMatrix(samplingPoints[samplingIndex].constData());
+		glLoadMatrix(samplingPoints[0].constData());
 		glTranslatef(-(GLfloat)center.X, -(GLfloat)center.Y, -(GLfloat)center.Z);
-
 		atomRenderer.renderOffscreen(false, projMatrix, QSize(renderBufferSize, renderBufferSize));
-
-		// Flush the buffer to force drawing of all objects thus far.
 		CHECK_OPENGL(glFlush());
-		if(useOnscreenRendering)
-			renderingWindow->swapBuffers();
+		pbuffer->toImage();
 
-		if(useOnscreenRendering || renderingWindow->hasFrameBufferExtension()) {
-			CHECK_OPENGL(glReadPixels(0, 0, renderBufferSize, renderBufferSize, GL_RGBA, GL_UNSIGNED_BYTE, pixelData.get()));
+		// Now do the actual rendering passes.
+		for(size_t samplingIndex=0; samplingIndex<numSamplingPoints; samplingIndex++) {
+			if(progress.isCanceled()) break;
+			progress.setValue(samplingIndex);
 
-			//QImage image((const uchar*)pixelData.get(), renderBufferSize, renderBufferSize, QImage::Format_ARGB32);
+			CHECK_OPENGL(glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT));
+
+			glLoadMatrix(samplingPoints[samplingIndex].constData());
+			glTranslatef(-(GLfloat)center.X, -(GLfloat)center.Y, -(GLfloat)center.Z);
+
+			atomRenderer.renderOffscreen(false, projMatrix, QSize(renderBufferSize, renderBufferSize));
+
+			// Flush the buffer to force drawing of all objects send into the OpenGL pipeline thus far.
+			CHECK_OPENGL(glFlush());
+
+			QImage image = pbuffer->toImage();
 			//image.save(QString("lighting_%1.png").arg(samplingIndex));
-
-			const quint32* p = pixelData.get();
-			for(size_t i=renderBufferSize*renderBufferSize; i--; ++p) {
-				quint32 index = (*p) & 0xFFFFFF;
-				OVITO_ASSERT(index < numAtoms || index == 0xFFFFFF);
-				if(index != 0xFFFFFF) ambientArray[index]++;
+			
+			for(int y = 0; y < renderBufferSize; y++) {
+				for(int x = 0; x < renderBufferSize; x++) {
+					QRgb pixel = image.pixel(x,y);
+					unsigned int index = (qRed(pixel) << 0) | (qGreen(pixel) << 8) | (qBlue(pixel) << 16);
+					if(index >= numAtoms && index != 0xFFFFFF) {
+						throw Exception(tr("Calculation failed. OpenGL off-screen pixel buffer contains invalid data."));
+					}
+					if(index != 0xFFFFFF) ambientArray[index]++;
+				}
 			}
 		}
 	}
@@ -477,25 +417,11 @@ EvaluationStatus AmbientLightingModifier::doAnalysis(TimeTicks time, bool suppre
 
 	// Restore old settings.
 	CHECK_OPENGL(glPopAttrib());
-
-	if(!useOnscreenRendering && renderingWindow->hasFrameBufferExtension()) {
-		// Make the normal window the target again.
-		renderingWindow->glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
-
-		// Clean up
-		renderingWindow->glDeleteFramebuffersEXT(1, &frameBuffer);
-		renderingWindow->glDeleteRenderbuffersEXT(1, &renderBuffer);
-		renderingWindow->glDeleteRenderbuffersEXT(1, &depthRenderBuffer);
 	}
 
-	}
-
-	if(useOnscreenRendering)
-		delete renderingContainer;
-
 	// Determine the maximum ambient value.
+	// The values of all atoms will be normalized by this maximum.
 	maxAmbient = 0;
-
 	QVector<unsigned int>::const_iterator ambientp = ambientArray.begin();
 	for(; ambientp != ambientArray.end(); ++ambientp)
 		if(*ambientp > maxAmbient) maxAmbient = *ambientp;
@@ -560,35 +486,36 @@ void AmbientLightingModifierEditor::createUI(const RolloutInsertionParameters& r
     // Create the rollout contents.
 	QVBoxLayout* layout1 = new QVBoxLayout(rollout);
 	layout1->setContentsMargins(4,4,4,4);
+#ifndef Q_WS_MAC
 	layout1->setSpacing(0);
+#endif
 
 	QGridLayout* layout2 = new QGridLayout();
 	layout2->setContentsMargins(0,0,0,0);
+#ifndef Q_WS_MAC
 	layout2->setSpacing(0);
+#endif
 	layout2->setColumnStretch(1, 1);
 	layout1->addLayout(layout2);
 
 	// Intensity parameter.
 	FloatControllerUI* intensityPUI = new FloatControllerUI(this, PROPERTY_FIELD_DESCRIPTOR(AmbientLightingModifier, _intensity));
 	layout2->addWidget(intensityPUI->label(), 0, 0);
-	layout2->addWidget(intensityPUI->textBox(), 0, 1);
-	layout2->addWidget(intensityPUI->spinner(), 0, 2);
+	layout2->addLayout(intensityPUI->createFieldLayout(), 0, 1);
 	intensityPUI->setMinValue(0);
 	intensityPUI->setMaxValue(1);
 
 	// Sampling level parameter.
 	IntegerPropertyUI* samplingLevelPUI = new IntegerPropertyUI(this, PROPERTY_FIELD_DESCRIPTOR(AmbientLightingModifier, _samplingLevel));
 	layout2->addWidget(samplingLevelPUI->label(), 1, 0);
-	layout2->addWidget(samplingLevelPUI->textBox(), 1, 1);
-	layout2->addWidget(samplingLevelPUI->spinner(), 1, 2);
+	layout2->addLayout(samplingLevelPUI->createFieldLayout(), 1, 1);
 	samplingLevelPUI->setMinValue(0);
 	samplingLevelPUI->setMaxValue(4);
 
 	// Buffer resolution parameter.
 	IntegerPropertyUI* bufferResPUI = new IntegerPropertyUI(this, PROPERTY_FIELD_DESCRIPTOR(AmbientLightingModifier, _bufferResolution));
 	layout2->addWidget(bufferResPUI->label(), 2, 0);
-	layout2->addWidget(bufferResPUI->textBox(), 2, 1);
-	layout2->addWidget(bufferResPUI->spinner(), 2, 2);
+	layout2->addLayout(bufferResPUI->createFieldLayout(), 2, 1);
 	bufferResPUI->setMinValue(0);
 	bufferResPUI->setMaxValue(5);
 
