@@ -66,7 +66,7 @@ CommonNeighborAnalysisModifier::CommonNeighborAnalysisModifier(bool isLoading)
 
 		intrusive_ptr<AtomType> icosahedralType(new AtomType());
 		icosahedralType->setName(tr("Icosahedral"));
-		icosahedralType->setColor(Color(1.0f, 1.0f, 0.7f));
+		icosahedralType->setColor(Color(0.2f, 1.0f, 1.0f));
 		_atomTypesList.push_back(icosahedralType);
 
 		intrusive_ptr<AtomType> noneType(new AtomType());
@@ -174,18 +174,17 @@ bool CommonNeighborAnalysisModifier::calculate(AtomsObject* atomsObject, bool su
 {
 	ProgressIndicator progress(tr("Performing common neighbor analysis (on %n processor(s))", NULL, QThread::idealThreadCount()), atomsObject->atomsCount(), suppressDialogs);
 
-	// Build the nearest-neighbor list.
-	if(!buildNeighborList()) {
+	// Prepare the neighbor list.
+	OnTheFlyNeighborList neighborList(nearestNeighborList()->nearestNeighborCutoff());
+	if(!neighborList.prepare(atomsObject, suppressDialogs)) {
 		_cnaChannel->setSize(0);
 		return false;
 	}
-	if(!nearestNeighborList() || !nearestNeighborList()->isValid()) return false;
-	const NearestNeighborList& neighborList = *nearestNeighborList();
 
 	// Reserve output array memory.
 	_cnaChannel->setSize(atomsObject->atomsCount());
 
-	// Measure computation time.
+	// Measure analysis time.
 	QTime timer;
 	timer.start();
 
@@ -196,9 +195,6 @@ bool CommonNeighborAnalysisModifier::calculate(AtomsObject* atomsObject, bool su
 	QFuture<void> future = QtConcurrent::map(firstAtom, lastAtom, kernel);
 	progress.waitForFuture(future);
 
-	// Release neighbor list memory.
-	nearestNeighborList()->clear();
-
 	// Throw away results obtained so far if the user cancels the calculation.
 	if(future.isCanceled()) {
 		_cnaChannel->setSize(0);
@@ -211,73 +207,137 @@ bool CommonNeighborAnalysisModifier::calculate(AtomsObject* atomsObject, bool su
 	return true;
 }
 
-/// A pair of atoms that are near-neighbors.
-typedef pair<NearestNeighborList::NeighborListAtom*, NearestNeighborList::NeighborListAtom*> Bond;
+#define CNA_MAX_PATTERN_NEIGHBORS 20
 
-void findCommonNeighbors(const NearestNeighborList::NeighborListAtom* atom1, const NearestNeighborList::NeighborListAtom* atom2, NearestNeighborList::NeighborList& commonNeighbors)
+/// Pair of neighbor atoms that form a bond (bit-wise storage).
+typedef unsigned int CNAPairBond;
+
+/**
+ * A bit-flag array indicating which pairs of neighbors are bonded
+ * and which are not.
+ */
+struct NeighborBondArray
 {
-	for(int ni1 = atom1->neighbors.size(); ni1--; ) {
-		NearestNeighborList::NeighborListAtom* n1 = atom1->neighbors[ni1];
-		for(int ni2 = atom2->neighbors.size(); ni2--; ) {
-			if(n1 == atom2->neighbors[ni2]) {
-				commonNeighbors.append(n1);
+	/// Default constructor.
+	NeighborBondArray() {
+		memset(neighborArray, 0, sizeof(neighborArray));
+	}
+
+	/// Two-dimensional bit array that stores the bonds between neighbors.
+	unsigned int neighborArray[CNA_MAX_PATTERN_NEIGHBORS];
+
+	/// Returns whether two nearest neighbors have a bond between them.
+	inline bool neighborBond(int neighborIndex1, int neighborIndex2) const {
+		OVITO_ASSERT(neighborIndex1 < CNA_MAX_PATTERN_NEIGHBORS);
+		OVITO_ASSERT(neighborIndex2 < CNA_MAX_PATTERN_NEIGHBORS);
+		return (neighborArray[neighborIndex1] & (1<<neighborIndex2));
+	}
+
+	/// Sets whether two nearest neighbors have a bond between them.
+	void setNeighborBond(int neighborIndex1, int neighborIndex2, bool bonded) {
+		OVITO_ASSERT(neighborIndex1 < CNA_MAX_PATTERN_NEIGHBORS);
+		OVITO_ASSERT(neighborIndex2 < CNA_MAX_PATTERN_NEIGHBORS);
+		if(bonded) {
+			neighborArray[neighborIndex1] |= (1<<neighborIndex2);
+			neighborArray[neighborIndex2] |= (1<<neighborIndex1);
+		}
+		else {
+			neighborArray[neighborIndex1] &= ~(1<<neighborIndex2);
+			neighborArray[neighborIndex2] &= ~(1<<neighborIndex1);
+		}
+	}
+};
+
+/******************************************************************************
+* Find all atoms that are nearest neighbors of the given pair of atoms.
+******************************************************************************/
+static int findCommonNeighbors(const NeighborBondArray& neighborArray, int neighborIndex, unsigned int& commonNeighbors, int numNeighbors)
+{
+	commonNeighbors = neighborArray.neighborArray[neighborIndex];
+#ifndef Q_CC_MSVC
+	// Count the number of bits set in neighbor bit field.
+	return __builtin_popcount(commonNeighbors);
+#else
+	// Count the number of bits set in neighbor bit field.
+	unsigned int v = commonNeighbors - ((commonNeighbors >> 1) & 0x55555555);
+	v = (v & 0x33333333) + ((v >> 2) & 0x33333333);
+	return ((v + (v >> 4) & 0xF0F0F0F) * 0x1010101) >> 24;
+#endif
+}
+
+/******************************************************************************
+* Finds all bonds between common nearest neighbors.
+******************************************************************************/
+static int findNeighborBonds(const NeighborBondArray& neighborArray, unsigned int commonNeighbors, int numNeighbors, CNAPairBond* neighborBonds)
+{
+	int numBonds = 0;
+
+	unsigned int nib[CNA_MAX_PATTERN_NEIGHBORS];
+	int nibn = 0;
+	unsigned int ni1b = 1;
+	for(int ni1 = 0; ni1 < numNeighbors; ni1++, ni1b <<= 1) {
+		if(commonNeighbors & ni1b) {
+			unsigned int b = commonNeighbors & neighborArray.neighborArray[ni1];
+			for(int n = 0; n < nibn; n++) {
+				if(b & nib[n]) {
+					OVITO_ASSERT(numBonds < CNA_MAX_PATTERN_NEIGHBORS*CNA_MAX_PATTERN_NEIGHBORS);
+					neighborBonds[numBonds++] = ni1b | nib[n];
+				}
 			}
+
+			nib[nibn++] = ni1b;
 		}
 	}
+	return numBonds;
 }
 
-void findNeighborBonds(const NearestNeighborList::NeighborListAtom* atom1, const NearestNeighborList::NeighborListAtom* atom2, const NearestNeighborList::NeighborList& commonNeighbors, vector<Bond>& neighborBonds)
+/******************************************************************************
+* Find all chains of bonds.
+******************************************************************************/
+static int getAdjacentBonds(unsigned int atom, CNAPairBond* bondsToProcess, int& numBonds, unsigned int& atomsToProcess, unsigned int& atomsProcessed)
 {
-	for(int ni1 = commonNeighbors.size(); ni1--; ) {
-		NearestNeighborList::NeighborListAtom* n1 = commonNeighbors[ni1];
-		for(int ni2 = ni1; ni2--; ) {
-			NearestNeighborList::NeighborListAtom* n2 = commonNeighbors[ni2];
-			if(n1->hasNeighbor(n2))
-				neighborBonds.push_back(Bond(n1, n2));
-		}
-	}
-}
-
-int getAdjacentBonds(const NearestNeighborList::NeighborListAtom* atom, vector<Bond>& bondsToProcess, vector<const NearestNeighborList::NeighborListAtom*>& atomsToProcess, vector<const NearestNeighborList::NeighborListAtom*>& atomsProcessed)
-{
-    atomsProcessed.push_back(atom);
     int adjacentBonds = 0;
-	for(vector<Bond>::iterator bond = bondsToProcess.begin(); bond != bondsToProcess.end(); ) {
-		if(atom == bond->first || atom == bond->second) {
+	for(int b = numBonds - 1; b >= 0; b--) {
+		if(atom & *bondsToProcess) {
             ++adjacentBonds;
-
-    		if(find(atomsProcessed.begin(), atomsProcessed.end(), bond->first) == atomsProcessed.end()
-    			&& find(atomsToProcess.begin(), atomsToProcess.end(), bond->first) == atomsToProcess.end())
-    			atomsToProcess.push_back(bond->first);
-
-    		if(find(atomsProcessed.begin(), atomsProcessed.end(), bond->second) == atomsProcessed.end()
-    			&& find(atomsToProcess.begin(), atomsToProcess.end(), bond->second) == atomsToProcess.end())
-    			atomsToProcess.push_back(bond->second);
-
-            bond = bondsToProcess.erase(bond);
+   			atomsToProcess |= *bondsToProcess & (~atomsProcessed);
+   			memmove(bondsToProcess, bondsToProcess + 1, sizeof(CNAPairBond) * b);
+   			numBonds--;
 		}
-		else ++bond;
+		else ++bondsToProcess;
 	}
 	return adjacentBonds;
 }
 
-int calcMaxChainLength(vector<Bond>& bondsToProcess)
+/******************************************************************************
+* Find all chains of bonds between common neighbors and determine the length
+* of the longest continuous chain.
+******************************************************************************/
+static int calcMaxChainLength(CNAPairBond* neighborBonds, int numBonds)
 {
     // Group the common bonds into clusters.
 	int maxChainLength = 0;
-	while(!bondsToProcess.empty()) {
+	while(numBonds) {
         // Make a new cluster starting with the first remaining bond to be processed.
-		Bond newBond = bondsToProcess.back();
-		bondsToProcess.pop_back();
-        vector<const NearestNeighborList::NeighborListAtom*> atomsToProcess, atomsProcessed;
-		atomsToProcess.push_back(newBond.first);
-		atomsToProcess.push_back(newBond.second);
+		numBonds--;
+        unsigned int atomsToProcess = neighborBonds[numBonds];
+        unsigned int atomsProcessed = 0;
 		int clusterSize = 1;
-        while(!atomsToProcess.empty()) {
-			const NearestNeighborList::NeighborListAtom* nextAtom = atomsToProcess.back();
-			atomsToProcess.pop_back();
-			clusterSize += getAdjacentBonds(nextAtom, bondsToProcess, atomsToProcess, atomsProcessed);
+        do {
+#ifndef Q_CC_MSVC
+        	// Determine the number of trailing 0-bits in atomsToProcess, starting at the least significant bit position.
+			int nextAtomIndex = __builtin_ctz(atomsToProcess);
+#else
+			unsigned long nextAtomIndex;
+			_BitScanForward(&nextAtomIndex, atomsToProcess);
+			OVITO_ASSERT(nextAtomIndex >= 0 && nextAtomIndex < 32);
+#endif
+			unsigned int nextAtom = 1 << nextAtomIndex;
+        	atomsProcessed |= nextAtom;
+			atomsToProcess &= ~nextAtom;
+			clusterSize += getAdjacentBonds(nextAtom, neighborBonds, numBonds, atomsToProcess, atomsProcessed);
 		}
+        while(atomsToProcess);
         if(clusterSize > maxChainLength)
         	maxChainLength = clusterSize;
 	}
@@ -292,157 +352,203 @@ void CommonNeighborAnalysisModifier::Kernel::operator()(int atomIndex)
 	int& result = atomTypeIndices.dataInt()[atomIndex];
 	result = OTHER; // Start with assuming that the CNA type is unknown.
 
-	const NearestNeighborList::NeighborListAtom& atom1 = nnlist.neighborLists()[atomIndex];
+	// Store neighbor vectors in a local array.
+	int numNeighbors = 0;
+	Vector3 neighborVectors[CNA_MAX_PATTERN_NEIGHBORS];
+	for(OnTheFlyNeighborList::iterator neighborIter(nnlist, atomIndex); !neighborIter.atEnd(); neighborIter.next()) {
+		if(numNeighbors == CNA_MAX_PATTERN_NEIGHBORS) return;
+		neighborVectors[numNeighbors] = neighborIter.delta();
+		numNeighbors++;
+	}
 
-	if(atom1.neighbors.size() == 12) { // Detect FCC and HCP atoms each having 12 NN.
-		size_t n421 = 0;
-		size_t n422 = 0;
-		size_t n552 = 0;
-		for(int ni2 = atom1.neighbors.size(); ni2--; ) {
-			const NearestNeighborList::NeighborListAtom* atom2 = atom1.neighbors[ni2];
+	if(numNeighbors == 12) { // Detect FCC and HCP atoms each having 12 NN.
 
-			// Determine number of neighbors the two pair atoms have in common.
-			NearestNeighborList::NeighborList commonNeighbors;
-			findCommonNeighbors(&atom1, atom2, commonNeighbors);
-			if(commonNeighbors.size() != 4 && commonNeighbors.size() != 5) return;
+		// Compute bond bit-flag array.
+		NeighborBondArray neighborArray;
+		for(int ni1 = 0; ni1 < 12; ni1++) {
+			neighborArray.setNeighborBond(ni1, ni1, false);
+			for(int ni2 = ni1+1; ni2 < 12; ni2++)
+				neighborArray.setNeighborBond(ni1, ni2, LengthSquared(neighborVectors[ni1] - neighborVectors[ni2]) <= nnlist.cutoffRadiusSquared());
+		}
 
-			// Determine the number of bonds among the shared neighbors.
-			vector<Bond> neighborBonds;
-			findNeighborBonds(&atom1, atom2, commonNeighbors, neighborBonds);
-			if(neighborBonds.size() != 2 && neighborBonds.size() != 5) return;
-			int numNeighborBonds = neighborBonds.size();
+		int n421 = 0;
+		int n422 = 0;
+		int n555 = 0;
+		for(int ni = 0; ni < 12; ni++) {
+
+			// Determine number of neighbors the two atoms have in common.
+			unsigned int commonNeighbors;
+			int numCommonNeighbors = findCommonNeighbors(neighborArray, ni, commonNeighbors, 12);
+			if(numCommonNeighbors != 4 && numCommonNeighbors != 5)
+				return;
+
+			// Determine the number of bonds among the common neighbors.
+			CNAPairBond neighborBonds[CNA_MAX_PATTERN_NEIGHBORS*CNA_MAX_PATTERN_NEIGHBORS];
+			int numNeighborBonds = findNeighborBonds(neighborArray, commonNeighbors, 12, neighborBonds);
+			if(numNeighborBonds != 2 && numNeighborBonds != 5)
+				break;
 
 			// Determine the number of bonds in the longest continuous chain.
-			int maxChainLength = calcMaxChainLength(neighborBonds);
-			if(commonNeighbors.size() == 4 && numNeighborBonds == 2) {
+			int maxChainLength = calcMaxChainLength(neighborBonds, numNeighborBonds);
+			if(numCommonNeighbors == 4 && numNeighborBonds == 2) {
 				if(maxChainLength == 1) n421++;
 				else if(maxChainLength == 2) n422++;
 				else return;
 			}
-			else if(commonNeighbors.size() == 5 && numNeighborBonds == 5 && maxChainLength == 2) n552++;
+			else if(numCommonNeighbors == 5 && numNeighborBonds == 5 && maxChainLength == 5) n555++;
 			else return;
 		}
 		if(n421 == 12) result = FCC;
 		else if(n421 == 6 && n422 == 6) result = HCP;
-		else if(n552 == 12) result = ICOSAHEDRAL;
+		else if(n555 == 12) result = ICOSAHEDRAL;
 	}
-	else if(atom1.neighbors.size() == 14) { // Detect BCC atoms having 14 NN (in 1st and 2nd shell).
-		size_t n444 = 0;
-		size_t n555 = 0;
-		size_t n666 = 0;
-		for(int ni2 = atom1.neighbors.size(); ni2--; ) {
-			const NearestNeighborList::NeighborListAtom* atom2 = atom1.neighbors[ni2];
+	else if(numNeighbors == 14) { // Detect BCC atoms having 14 NN (in 1st and 2nd shell).
 
-			// Determine number of neighbors the two pair atoms have in common.
-			NearestNeighborList::NeighborList commonNeighbors;
-			findCommonNeighbors(&atom1, atom2, commonNeighbors);
-			if(commonNeighbors.size() != 4 && commonNeighbors.size() != 5 && commonNeighbors.size() != 6) return;
+		// Compute bond bit-flag array.
+		NeighborBondArray neighborArray;
+		for(int ni1 = 0; ni1 < 14; ni1++) {
+			neighborArray.setNeighborBond(ni1, ni1, false);
+			for(int ni2 = ni1+1; ni2 < 14; ni2++)
+				neighborArray.setNeighborBond(ni1, ni2, LengthSquared(neighborVectors[ni1] - neighborVectors[ni2]) <= nnlist.cutoffRadiusSquared());
+		}
 
-			// Determine the number of bonds among the shared neighbors.
-			vector<Bond> neighborBonds;
-			findNeighborBonds(&atom1, atom2, commonNeighbors, neighborBonds);
-			if(neighborBonds.size() != 4 && neighborBonds.size() != 5 && neighborBonds.size() != 6) return;
-			int numNeighborBonds = neighborBonds.size();
+		int n444 = 0;
+		int n555 = 0;
+		int n666 = 0;
+		for(int ni = 0; ni < 14; ni++) {
+
+			// Determine number of neighbors the two atoms have in common.
+			unsigned int commonNeighbors;
+			int numCommonNeighbors = findCommonNeighbors(neighborArray, ni, commonNeighbors, 14);
+			if(numCommonNeighbors != 4 && numCommonNeighbors != 5 && numCommonNeighbors != 6)
+				return;
+
+			// Determine the number of bonds among the common neighbors.
+			CNAPairBond neighborBonds[CNA_MAX_PATTERN_NEIGHBORS*CNA_MAX_PATTERN_NEIGHBORS];
+			int numNeighborBonds = findNeighborBonds(neighborArray, commonNeighbors, 14, neighborBonds);
+			if(numNeighborBonds != 4 && numNeighborBonds != 5 && numNeighborBonds != 6)
+				break;
 
 			// Determine the number of bonds in the longest continuous chain.
-			int maxChainLength = calcMaxChainLength(neighborBonds);
-			if(commonNeighbors.size() == 4 && numNeighborBonds == 4 && maxChainLength == 4) n444++;
-			else if(commonNeighbors.size() == 6 && numNeighborBonds == 6 && maxChainLength == 6) n666++;
-			else if(commonNeighbors.size() == 5 && numNeighborBonds == 5 && maxChainLength == 5) n555++;
+			int maxChainLength = calcMaxChainLength(neighborBonds, numNeighborBonds);
+			if(numCommonNeighbors == 4 && numNeighborBonds == 4 && maxChainLength == 4) n444++;
+			else if(numCommonNeighbors == 6 && numNeighborBonds == 6 && maxChainLength == 6) n666++;
+			else if(numCommonNeighbors == 5 && numNeighborBonds == 5 && maxChainLength == 5) n555++;
 			else return;
 		}
 		if(n666 == 8 && n444 == 6) result = BCC;
 		else if(n666 == 6 && n444 == 4 && n555 == 4) result = BCC_TWIN;
 	}
-	else if(atom1.neighbors.size() == 9) { // Detect cg-N atoms having 9 NN.
-		size_t n211 = 0;
-		size_t n421 = 0;
-		for(int ni2 = atom1.neighbors.size(); ni2--; ) {
-			const NearestNeighborList::NeighborListAtom* atom2 = atom1.neighbors[ni2];
-			// Determine number of neighbors the two pair atoms have in common.
-			NearestNeighborList::NeighborList commonNeighbors;
-			findCommonNeighbors(&atom1, atom2, commonNeighbors);
-			if(commonNeighbors.size() != 2 && commonNeighbors.size() != 4) return;
+	else if(numNeighbors == 9) { // Detect cg-N atoms having 9 NN.
 
-			// Determine the number of bonds among the shared neighbors.
-			vector<Bond> neighborBonds;
-			findNeighborBonds(&atom1, atom2, commonNeighbors, neighborBonds);
-			if(neighborBonds.size() != 1 && neighborBonds.size() != 2) return;
-			int numNeighborBonds = neighborBonds.size();
+		// Compute bond bit-flag array.
+		NeighborBondArray neighborArray;
+		for(int ni1 = 0; ni1 < 9; ni1++) {
+			neighborArray.setNeighborBond(ni1, ni1, false);
+			for(int ni2 = ni1+1; ni2 < 9; ni2++)
+				neighborArray.setNeighborBond(ni1, ni2, LengthSquared(neighborVectors[ni1] - neighborVectors[ni2]) <= nnlist.cutoffRadiusSquared());
+		}
+
+		int n211 = 0;
+		int n421 = 0;
+		for(int ni = 0; ni < 9; ni++) {
+
+			// Determine number of neighbors the two atoms have in common.
+			unsigned int commonNeighbors;
+			int numCommonNeighbors = findCommonNeighbors(neighborArray, ni, commonNeighbors, 9);
+			if(numCommonNeighbors != 2 && numCommonNeighbors != 4)
+				return;
+
+			// Determine the number of bonds among the common neighbors.
+			CNAPairBond neighborBonds[CNA_MAX_PATTERN_NEIGHBORS*CNA_MAX_PATTERN_NEIGHBORS];
+			int numNeighborBonds = findNeighborBonds(neighborArray, commonNeighbors, 9, neighborBonds);
+			if(numNeighborBonds != 1 && numNeighborBonds != 2)
+				break;
 
 			// Determine the number of bonds in the longest continuous chain.
-			int maxChainLength = calcMaxChainLength(neighborBonds);
-			if(commonNeighbors.size() == 2 && numNeighborBonds == 1 && maxChainLength == 1) n211++;
-			else if(commonNeighbors.size() == 4 && numNeighborBonds == 2 && maxChainLength == 1) n421++;
+			int maxChainLength = calcMaxChainLength(neighborBonds, numNeighborBonds);
+			if(numCommonNeighbors == 2 && numNeighborBonds == 1 && maxChainLength == 1) n211++;
+			else if(numCommonNeighbors == 4 && numNeighborBonds == 2 && maxChainLength == 1) n421++;
 			else return;
 		}
 		if(n211 == 6 && n421 == 3) result = CG_N;
 	}
-	else if(atom1.neighbors.size() == 16) { // Detect diamond atoms having 16 NN.
-		size_t n543 = 0;
-		size_t n555 = 0;
-		size_t n663 = 0;
-		size_t n677 = 0;
-		size_t n699 = 0;
-		size_t n799 = 0;
-		for(int ni2 = atom1.neighbors.size(); ni2--; ) {
-			const NearestNeighborList::NeighborListAtom* atom2 = atom1.neighbors[ni2];
+	else if(numNeighbors == 16) { // Detect diamond atoms having 16 NN.
 
-			// Determine number of neighbors the two pair atoms have in common.
-			NearestNeighborList::NeighborList commonNeighbors;
-			findCommonNeighbors(&atom1, atom2, commonNeighbors);
-			if(commonNeighbors.size() < 5 || commonNeighbors.size() > 7) return;
+		// Compute bond bit-flag array.
+		NeighborBondArray neighborArray;
+		for(int ni1 = 0; ni1 < 16; ni1++) {
+			neighborArray.setNeighborBond(ni1, ni1, false);
+			for(int ni2 = ni1+1; ni2 < 16; ni2++)
+				neighborArray.setNeighborBond(ni1, ni2, LengthSquared(neighborVectors[ni1] - neighborVectors[ni2]) <= nnlist.cutoffRadiusSquared());
+		}
+		int n543 = 0;
+		int n555 = 0;
+		int n663 = 0;
+		int n677 = 0;
+		int n699 = 0;
+		int n799 = 0;
+		for(int ni = 0; ni < 16; ni++) {
 
-			// Determine the number of bonds among the shared neighbors.
-			vector<Bond> neighborBonds;
-			findNeighborBonds(&atom1, atom2, commonNeighbors, neighborBonds);
-			if(neighborBonds.size() < 4 || neighborBonds.size() > 9) return;
-			int numNeighborBonds = neighborBonds.size();
+			// Determine number of neighbors the two atoms have in common.
+			unsigned int commonNeighbors;
+			int numCommonNeighbors = findCommonNeighbors(neighborArray, ni, commonNeighbors, 16);
+			if(numCommonNeighbors < 5 || numCommonNeighbors > 7) return;
+
+			// Determine the number of bonds among the common neighbors.
+			CNAPairBond neighborBonds[CNA_MAX_PATTERN_NEIGHBORS*CNA_MAX_PATTERN_NEIGHBORS];
+			int numNeighborBonds = findNeighborBonds(neighborArray, commonNeighbors, 16, neighborBonds);
+			if(numNeighborBonds < 4 || numNeighborBonds > 9) break;
 
 			// Determine the number of bonds in the longest continuous chain.
-			int maxChainLength = calcMaxChainLength(neighborBonds);
-			if(commonNeighbors.size() == 5 && numNeighborBonds == 4 && maxChainLength == 3) n543++;
-			else if(commonNeighbors.size() == 6 && numNeighborBonds == 6 && maxChainLength == 3) n663++;
-			else if(commonNeighbors.size() == 6 && numNeighborBonds == 7 && maxChainLength == 7) n677++;
-			else if(commonNeighbors.size() == 5 && numNeighborBonds == 5 && maxChainLength == 5) n555++;
-			else if(commonNeighbors.size() == 6 && numNeighborBonds == 9 && maxChainLength == 9) n699++;
-			else if(commonNeighbors.size() == 7 && numNeighborBonds == 9 && maxChainLength == 9) n799++;
+			int maxChainLength = calcMaxChainLength(neighborBonds, numNeighborBonds);
+			if(numCommonNeighbors == 5 && numNeighborBonds == 4 && maxChainLength == 3) n543++;
+			else if(numCommonNeighbors == 6 && numNeighborBonds == 6 && maxChainLength == 3) n663++;
+			else if(numCommonNeighbors == 6 && numNeighborBonds == 7 && maxChainLength == 7) n677++;
+			else if(numCommonNeighbors == 5 && numNeighborBonds == 5 && maxChainLength == 5) n555++;
+			else if(numCommonNeighbors == 6 && numNeighborBonds == 9 && maxChainLength == 9) n699++;
+			else if(numCommonNeighbors == 7 && numNeighborBonds == 9 && maxChainLength == 9) n799++;
 			else return;
 		}
 		if(n543 == 12 && n663 == 4) result = DIAMOND;
 		else if(n555 == 6 && n677 == 3 && n699 == 1 && n543 == 3 && n799 == 3) result = DIAMOND_STACKINGFAULT;
 	}
-	else if(atom1.neighbors.size() == 17) { // Detect hexagonal diamond atoms having 17 NN.
-		size_t n677 = 0;
-		size_t n555 = 0;
-		size_t n81212 = 0;
-		size_t n699 = 0;
-		size_t n543 = 0;
-		size_t n799 = 0;
-		size_t n663 = 0;
-		for(int ni2 = atom1.neighbors.size(); ni2--; ) {
-			const NearestNeighborList::NeighborListAtom* atom2 = atom1.neighbors[ni2];
+	else if(numNeighbors == 17) { // Detect hexagonal diamond atoms having 17 NN.
+		// Compute bond bit-flag array.
+		NeighborBondArray neighborArray;
+		for(int ni1 = 0; ni1 < 17; ni1++) {
+			neighborArray.setNeighborBond(ni1, ni1, false);
+			for(int ni2 = ni1+1; ni2 < 17; ni2++)
+				neighborArray.setNeighborBond(ni1, ni2, LengthSquared(neighborVectors[ni1] - neighborVectors[ni2]) <= nnlist.cutoffRadiusSquared());
+		}
+		int n677 = 0;
+		int n555 = 0;
+		int n81212 = 0;
+		int n699 = 0;
+		int n543 = 0;
+		int n799 = 0;
+		int n663 = 0;
+		for(int ni = 0; ni < 17; ni++) {
 
-			// Determine number of neighbors the two pair atoms have in common.
-			NearestNeighborList::NeighborList commonNeighbors;
-			findCommonNeighbors(&atom1, atom2, commonNeighbors);
-			if(commonNeighbors.size() < 5 || commonNeighbors.size() > 8) return;
+			// Determine number of neighbors the two atoms have in common.
+			unsigned int commonNeighbors;
+			int numCommonNeighbors = findCommonNeighbors(neighborArray, ni, commonNeighbors, 17);
+			if(numCommonNeighbors < 5 || numCommonNeighbors > 8) return;
 
-			// Determine the number of bonds among the shared neighbors.
-			vector<Bond> neighborBonds;
-			findNeighborBonds(&atom1, atom2, commonNeighbors, neighborBonds);
-			if(neighborBonds.size() < 4 || neighborBonds.size() > 12) return;
-			int numNeighborBonds = neighborBonds.size();
+			// Determine the number of bonds among the common neighbors.
+			CNAPairBond neighborBonds[CNA_MAX_PATTERN_NEIGHBORS*CNA_MAX_PATTERN_NEIGHBORS];
+			int numNeighborBonds = findNeighborBonds(neighborArray, commonNeighbors, 17, neighborBonds);
+			if(numNeighborBonds < 4 || numNeighborBonds > 12) break;
 
 			// Determine the number of bonds in the longest continuous chain.
-			int maxChainLength = calcMaxChainLength(neighborBonds);
-			if(commonNeighbors.size() == 5 && numNeighborBonds == 4 && maxChainLength == 3) n543++;
-			else if(commonNeighbors.size() == 6 && numNeighborBonds == 7 && maxChainLength == 7) n677++;
-			else if(commonNeighbors.size() == 6 && numNeighborBonds == 6 && maxChainLength == 3) n663++;
-			else if(commonNeighbors.size() == 5 && numNeighborBonds == 5 && maxChainLength == 5) n555++;
-			else if(commonNeighbors.size() == 8 && numNeighborBonds == 12 && maxChainLength == 12) n81212++;
-			else if(commonNeighbors.size() == 6 && numNeighborBonds == 9 && maxChainLength == 9) n699++;
-			else if(commonNeighbors.size() == 7 && numNeighborBonds == 9 && maxChainLength == 9) n799++;
+			int maxChainLength = calcMaxChainLength(neighborBonds, numNeighborBonds);
+			if(numCommonNeighbors == 5 && numNeighborBonds == 4 && maxChainLength == 3) n543++;
+			else if(numCommonNeighbors == 6 && numNeighborBonds == 7 && maxChainLength == 7) n677++;
+			else if(numCommonNeighbors == 6 && numNeighborBonds == 6 && maxChainLength == 3) n663++;
+			else if(numCommonNeighbors == 5 && numNeighborBonds == 5 && maxChainLength == 5) n555++;
+			else if(numCommonNeighbors == 8 && numNeighborBonds == 12 && maxChainLength == 12) n81212++;
+			else if(numCommonNeighbors == 6 && numNeighborBonds == 9 && maxChainLength == 9) n699++;
+			else if(numCommonNeighbors == 7 && numNeighborBonds == 9 && maxChainLength == 9) n799++;
 			else return;
 		}
 		if(n677 == 6 && n555 == 6 && n81212 == 3 && n699 == 2) result = HEX_DIAMOND;
@@ -463,7 +569,9 @@ void CommonNeighborAnalysisModifierEditor::createUI(const RolloutInsertionParame
     // Create the rollout contents.
 	QVBoxLayout* layout1 = new QVBoxLayout(rollout);
 	layout1->setContentsMargins(4,4,4,4);
+#ifndef Q_WS_MAC
 	layout1->setSpacing(0);
+#endif
 
 	BooleanPropertyUI* autoUpdateUI = new BooleanPropertyUI(this, PROPERTY_FIELD_DESCRIPTOR(AtomsObjectAnalyzerBase, _autoUpdateOnTimeChange));
 	layout1->addWidget(autoUpdateUI->checkBox());
@@ -485,7 +593,7 @@ void CommonNeighborAnalysisModifierEditor::createUI(const RolloutInsertionParame
 	class CustomRefTargetListParameterUI : public RefTargetListParameterUI {
 	public:
 		CustomRefTargetListParameterUI(PropertiesEditor* parentEditor, const PropertyFieldDescriptor& refField)
-			: RefTargetListParameterUI(parentEditor, refField) {}
+			: RefTargetListParameterUI(parentEditor, refField, RolloutInsertionParameters(), NULL, 260) {}
 	protected:
 		virtual QVariant getItemData(RefTarget* target, const QModelIndex& index, int role) {
 			if(role == Qt::DecorationRole && target != NULL) {
