@@ -212,7 +212,9 @@ void ColorCodingModifierEditor::createUI(const RolloutInsertionParameters& rollo
     // Create the rollout contents.
 	QVBoxLayout* layout1 = new QVBoxLayout(rollout);
 	layout1->setContentsMargins(4,4,4,4);
+#ifndef Q_WS_MAC
 	layout1->setSpacing(0);
+#endif
 
 	channelList = new QComboBox(rollout);
 	layout1->addWidget(new QLabel(tr("Data channel:"), rollout));
@@ -243,8 +245,7 @@ void ColorCodingModifierEditor::createUI(const RolloutInsertionParameters& rollo
 	// End value parameter.
 	FloatControllerUI* endValuePUI = new FloatControllerUI(this, PROPERTY_FIELD_DESCRIPTOR(ColorCodingModifier, endValueCtrl));
 	layout2->addWidget(endValuePUI->label(), 0, 0);
-	layout2->addWidget(endValuePUI->textBox(), 0, 1);
-	layout2->addWidget(endValuePUI->spinner(), 0, 2);
+	layout2->addLayout(endValuePUI->createFieldLayout(), 0, 1);
 
 	// Insert color legend display.
 	colorLegendLabel = new QLabel(rollout);
@@ -254,8 +255,7 @@ void ColorCodingModifierEditor::createUI(const RolloutInsertionParameters& rollo
 	// Start value parameter.
 	FloatControllerUI* startValuePUI = new FloatControllerUI(this, PROPERTY_FIELD_DESCRIPTOR(ColorCodingModifier, startValueCtrl));
 	layout2->addWidget(startValuePUI->label(), 2, 0);
-	layout2->addWidget(startValuePUI->textBox(), 2, 1);
-	layout2->addWidget(startValuePUI->spinner(), 2, 2);
+	layout2->addLayout(startValuePUI->createFieldLayout(), 2, 1);
 
 	layout1->addSpacing(8);
 	QPushButton* adjustBtn = new QPushButton(tr("Adjust range"), rollout);
