@@ -21,6 +21,7 @@
 
 #include <core/Core.h>
 #include <core/undo/UndoManager.h>
+#include <core/scene/objects/ModifiedObject.h>
 #include <atomviz/utils/muparser/muParser.h>
 
 #include "CreateExpressionChannelModifier.h"
@@ -77,11 +78,11 @@ void CreateExpressionChannelModifier::setDataChannelComponentCount(int newCompon
 /******************************************************************************
 * Determines the available variable names.
 ******************************************************************************/
-QStringList CreateExpressionChannelModifier::getVariableNames()
+QStringList CreateExpressionChannelModifier::getVariableNames(AtomsObject* inputObject)
 {
 	QStringList variableNames;
 
-	Q_FOREACH(DataChannel* channel, input()->dataChannels()) {
+	Q_FOREACH(DataChannel* channel, inputObject->dataChannels()) {
 
 		// Channels of custom data type are not supported by this modifier.
 		if(channel->type() != qMetaTypeId<int>() && channel->type() != qMetaTypeId<FloatType>()) continue;
@@ -101,7 +102,7 @@ QStringList CreateExpressionChannelModifier::getVariableNames()
 		}
 	}
 
-	if(input()->getStandardDataChannel(DataChannel::AtomIndexChannel) == NULL)
+	if(inputObject->getStandardDataChannel(DataChannel::AtomIndexChannel) == NULL)
 		variableNames << "AtomIndex";
 
 	return variableNames;
@@ -257,7 +258,7 @@ private:
 EvaluationStatus CreateExpressionChannelModifier::modifyAtomsObject(TimeTicks time, TimeInterval& validityInterval)
 {
 	// Get list of available input variables.
-	_variableNames = getVariableNames();
+	_variableNames = getVariableNames(input());
 
 	// Create and initialize the worker threads.
 	QVector<CreateExpressionEvaluationKernel> workers(max(QThread::idealThreadCount(), 1));
@@ -320,6 +321,21 @@ EvaluationStatus CreateExpressionChannelModifier::modifyAtomsObject(TimeTicks ti
 	return EvaluationStatus();
 }
 
+/******************************************************************************
+* This method is called by the system when the modifier has been inserted
+* into a ModifiedObject.
+******************************************************************************/
+void CreateExpressionChannelModifier::initializeModifier(ModifiedObject* modObject, ModifierApplication* modApp)
+{
+	// Get the list of variables from the input object.
+	PipelineFlowState input = modObject->evalObject(ANIM_MANAGER.time(), modApp, false);
+	AtomsObject* inputObject = dynamic_object_cast<AtomsObject>(input.result());
+	if(inputObject != NULL) {
+		// Get list of available input variables.
+		_variableNames = getVariableNames(inputObject);
+	}
+}
+
 IMPLEMENT_PLUGIN_CLASS(CreateExpressionChannelModifierEditor, AtomsObjectModifierEditorBase)
 
 /******************************************************************************
@@ -338,7 +354,9 @@ void CreateExpressionChannelModifierEditor::createUI(const RolloutInsertionParam
 	QGridLayout* channelPropertiesLayout = new QGridLayout(channelPropertiesGroupBox);
 	channelPropertiesLayout->setContentsMargins(4,4,4,4);
 	channelPropertiesLayout->setColumnStretch(1, 1);
+#ifndef Q_WS_MAC
 	channelPropertiesLayout->setSpacing(2);
+#endif
 
 	// Create the combo box with the standard data channel identifiers.
 	VariantComboBoxPropertyUI* dataChannelIdUI = new VariantComboBoxPropertyUI(this, "dataChannelId");
@@ -353,12 +371,12 @@ void CreateExpressionChannelModifierEditor::createUI(const RolloutInsertionParam
 	// Create the field with the data channel name.
 	dataChannelNameUI = new StringPropertyUI(this, "dataChannelName");
 	channelPropertiesLayout->addWidget(new QLabel(tr("Name:")), 1, 0);
-	channelPropertiesLayout->addWidget(dataChannelNameUI->textBox(), 1, 1, 1, 2);
+	channelPropertiesLayout->addWidget(dataChannelNameUI->textBox(), 1, 1);
 
 	// Create the combo box with the data channel types.
 	dataChannelTypeUI = new VariantComboBoxPropertyUI(this, "dataChannelDataType");
 	channelPropertiesLayout->addWidget(new QLabel(tr("Data type:")), 2, 0);
-	channelPropertiesLayout->addWidget(dataChannelTypeUI->comboBox(), 2, 1, 1, 2);
+	channelPropertiesLayout->addWidget(dataChannelTypeUI->comboBox(), 2, 1);
 	dataChannelTypeUI->comboBox()->addItem(tr("Float"), qMetaTypeId<FloatType>());
 	dataChannelTypeUI->comboBox()->addItem(tr("Integer"), qMetaTypeId<int>());
 
@@ -367,22 +385,23 @@ void CreateExpressionChannelModifierEditor::createUI(const RolloutInsertionParam
 	numComponentsUI->setMinValue(1);
 	numComponentsUI->setMaxValue(64);
 	channelPropertiesLayout->addWidget(new QLabel(tr("Number of components:")), 3, 0);
-	channelPropertiesLayout->addWidget(numComponentsUI->textBox(), 3, 1);
-	channelPropertiesLayout->addWidget(numComponentsUI->spinner(), 3, 2);
+	channelPropertiesLayout->addLayout(numComponentsUI->createFieldLayout(), 3, 1);
 
 	// Create the check box for the channel visibility flag.
 	BooleanPropertyUI* visibilityFlagUI = new BooleanPropertyUI(this, PROPERTY_FIELD_DESCRIPTOR(CreateExpressionChannelModifier, _dataChannelVisibility));
-	channelPropertiesLayout->addWidget(visibilityFlagUI->checkBox(), 4, 0, 1, 3);
+	channelPropertiesLayout->addWidget(visibilityFlagUI->checkBox(), 4, 0, 1, 2);
 
 	// Create the check box for the selection flag.
 	BooleanPropertyUI* selectionFlagUI = new BooleanPropertyUI(this, PROPERTY_FIELD_DESCRIPTOR(CreateExpressionChannelModifier, _onlySelectedAtoms));
-	channelPropertiesLayout->addWidget(selectionFlagUI->checkBox(), 5, 0, 1, 3);
+	channelPropertiesLayout->addWidget(selectionFlagUI->checkBox(), 5, 0, 1, 2);
 
 	QGroupBox* expressionsGroupBox = new QGroupBox(tr("Expressions"));
 	mainLayout->addWidget(expressionsGroupBox);
 	expressionsLayout = new QVBoxLayout(expressionsGroupBox);
 	expressionsLayout->setContentsMargins(4,4,4,4);
+#ifndef Q_WS_MAC
 	expressionsLayout->setSpacing(0);
+#endif
 
 	// Status label.
 	mainLayout->addWidget(statusLabel());
@@ -459,7 +478,7 @@ void CreateExpressionChannelModifierEditor::updateEditorFields()
 		labelText.append(QString("<li>%1</li>").arg(s));
 	}
 	labelText.append(QString("<li>N (number of atoms)</li>"));
-	labelText.append(QString("<li>t (current time frame)</li>"));
+	labelText.append(QString("<li>t (current animation frame)</li>"));
 	labelText.append("</ul><p></p>");
 	variableNamesList->setText(labelText);
 }
