@@ -26,6 +26,7 @@
 #include <core/scene/ObjectNode.h>
 #include <core/scene/animation/AnimManager.h>
 #include <core/scene/animation/controller/StandardControllers.h>
+#include <core/scene/objects/ModifiedObject.h>
 #include <core/gui/properties/FloatControllerUI.h>
 #include <core/gui/properties/VectorControllerUI.h>
 #include <core/gui/properties/BooleanPropertyUI.h>
@@ -135,8 +136,8 @@ EvaluationStatus SliceModifier::modifyAtomsObject(TimeTicks time, TimeInterval&
 		OVITO_ASSERT(output()->atomsCount() == input()->atomsCount() - numRejected);
 	}
 	else {
-		statusMessage += tr("\n%n atoms deleted", 0, numRejected);
-		statusMessage += tr("\n%n atoms remaining", 0, numKept);
+		statusMessage += tr("\n%n atoms selected", 0, numRejected);
+		statusMessage += tr("\n%n atoms unselected", 0, numKept);
 
 		// Get selection data channel.
 		DataChannel* selChannel = outputStandardChannel(DataChannel::SelectionChannel);
@@ -280,6 +281,25 @@ void SliceModifier::planeQuadIntersesction(const Ray3& r1, const Ray3& r2, const
 	}
 }
 
+/******************************************************************************
+* This method is called by the system when the modifier has been inserted
+* into a ModifiedObject.
+******************************************************************************/
+void SliceModifier::initializeModifier(ModifiedObject* modObject, ModifierApplication* modApp)
+{
+	// Get the simulation cell from the input object to center the slincing plane in
+	// the center of the simulation cell.
+	PipelineFlowState input = modObject->evalObject(ANIM_MANAGER.time(), modApp, false);
+	AtomsObject* inputObject = dynamic_object_cast<AtomsObject>(input.result());
+	if(inputObject != NULL) {
+		Point3 centerPoint = inputObject->simulationCell()->cellMatrix() * Point3(0.5, 0.5, 0.5);
+		FloatType centerDistance = DotProduct(normal(), centerPoint - ORIGIN);
+		if(fabs(centerDistance) > FLOATTYPE_EPSILON)
+			setDistance(centerDistance);
+	}
+}
+
+
 IMPLEMENT_PLUGIN_CLASS(SliceModifierEditor, AtomsObjectModifierEditorBase)
 
 /******************************************************************************
@@ -288,37 +308,40 @@ IMPLEMENT_PLUGIN_CLASS(SliceModifierEditor, AtomsObjectModifierEditorBase)
 void SliceModifierEditor::createUI(const RolloutInsertionParameters& rolloutParams)
 {
 	// Create a rollout.
-	QWidget* rollout = createRollout(tr("Slicing Plane"), rolloutParams, "atomviz.modifiers.slice");
+	QWidget* rollout = createRollout(tr("Slicing plane"), rolloutParams, "atomviz.modifiers.slice");
 
     // Create the rollout contents.
 	QGridLayout* layout = new QGridLayout(rollout);
 	layout->setContentsMargins(4,4,4,4);
+#ifndef Q_WS_MAC
 	layout->setHorizontalSpacing(0);
 	layout->setVerticalSpacing(2);
+#endif
 	layout->setColumnStretch(1, 1);
 
 	// Distance parameter.
 	FloatControllerUI* distancePUI = new FloatControllerUI(this, PROPERTY_FIELD_DESCRIPTOR(SliceModifier, _distanceCtrl));
 	layout->addWidget(distancePUI->label(), 0, 0);
-	layout->addWidget(distancePUI->textBox(), 0, 1);
-	layout->addWidget(distancePUI->spinner(), 0, 2);
+	layout->addLayout(distancePUI->createFieldLayout(), 0, 1);
 	distancePUI->setWhatsThis(tr("Specifies the distance of the plane to the origin of the simulation cell coordinate system. "
 							   "The distance is specified in the direction of the normal vector."));
 
 	// Normal parameter.
 	for(size_t i=0; i<3; i++) {
 		VectorControllerUI* normalPUI = new VectorControllerUI(this, PROPERTY_FIELD_DESCRIPTOR(SliceModifier, _normalCtrl), i);
+		normalPUI->label()->setTextFormat(Qt::RichText);
+		normalPUI->label()->setTextInteractionFlags(Qt::LinksAccessibleByMouse);
+		normalPUI->label()->setText(tr("<a href=\"%1\">%2</a>").arg(i).arg(normalPUI->label()->text()));
+		connect(normalPUI->label(), SIGNAL(linkActivated(const QString&)), this, SLOT(onXYZNormal(const QString&)));
 		layout->addWidget(normalPUI->label(), i+1, 0);
-		layout->addWidget(normalPUI->textBox(), i+1, 1);
-		layout->addWidget(normalPUI->spinner(), i+1, 2);
+		layout->addLayout(normalPUI->createFieldLayout(), i+1, 1);
 		normalPUI->setWhatsThis(tr("Specifies the components of the normal vector that defines the slicing plane orientation."));
 	}
 
 	// Slice width parameter.
 	FloatControllerUI* widthPUI = new FloatControllerUI(this, PROPERTY_FIELD_DESCRIPTOR(SliceModifier, _widthCtrl));
 	layout->addWidget(widthPUI->label(), 4, 0);
-	layout->addWidget(widthPUI->textBox(), 4, 1);
-	layout->addWidget(widthPUI->spinner(), 4, 2);
+	layout->addLayout(widthPUI->createFieldLayout(), 4, 1);
 	widthPUI->setMinValue(0);
 	widthPUI->setWhatsThis(tr("When this value is zero then all atoms on one side of the slicing plane are deleted--selected. "
 							"For a positive plane width all atoms whose distance to the plane is greater/smaller than "
@@ -326,27 +349,27 @@ void SliceModifierEditor::createUI(const RolloutInsertionParameters& rolloutPara
 
 	// Invert parameter.
 	BooleanPropertyUI* invertPUI = new BooleanPropertyUI(this, PROPERTY_FIELD_DESCRIPTOR(SliceModifier, _inverse));
-	layout->addWidget(invertPUI->checkBox(), 5, 0, 1, 3);
+	layout->addWidget(invertPUI->checkBox(), 5, 0, 1, 2);
 	invertPUI->setWhatsThis(tr("When this option is enabled the orientation of the slicing plane is reversed."));
 
 	// Create selection parameter.
 	BooleanPropertyUI* createSelectionPUI = new BooleanPropertyUI(this, PROPERTY_FIELD_DESCRIPTOR(SliceModifier, _createSelection));
-	layout->addWidget(createSelectionPUI->checkBox(), 6, 0, 1, 3);
+	layout->addWidget(createSelectionPUI->checkBox(), 6, 0, 1, 2);
 	createSelectionPUI->setWhatsThis(tr("Controls whether the atoms should be deleted or just selected."));
 
 	// Apply to selection only parameter.
 	BooleanPropertyUI* applyToSelectionPUI = new BooleanPropertyUI(this, PROPERTY_FIELD_DESCRIPTOR(SliceModifier, _applyToSelection));
-	layout->addWidget(applyToSelectionPUI->checkBox(), 7, 0, 1, 3);
+	layout->addWidget(applyToSelectionPUI->checkBox(), 7, 0, 1, 2);
 	applyToSelectionPUI->setWhatsThis(tr("When this option is enabled then the slicing modifier is only applied "
 									   "to the atoms that are currently selected. If there is no selection then all atoms will be affected by the modifier."));
 
 	// Add buttons for view alignment functions.
 	QPushButton* alignViewToPlaneBtn = new QPushButton(tr("Align view to plane"), rollout);
 	connect(alignViewToPlaneBtn, SIGNAL(clicked(bool)), this, SLOT(onAlignViewToPlane()));
-	layout->addWidget(alignViewToPlaneBtn, 8, 0, 1, 3);
+	layout->addWidget(alignViewToPlaneBtn, 8, 0, 1, 2);
 	QPushButton* alignPlaneToViewBtn = new QPushButton(tr("Align plane to view"), rollout);
 	connect(alignPlaneToViewBtn, SIGNAL(clicked(bool)), this, SLOT(onAlignPlaneToView()));
-	layout->addWidget(alignPlaneToViewBtn, 9, 0, 1, 3);
+	layout->addWidget(alignPlaneToViewBtn, 9, 0, 1, 2);
 
 	pickAtomPlaneInputMode = new PickAtomPlaneInputMode();
 	pickAtomPlaneInputModeAction = new ViewportModeAction("SliceModifier.AlignPlaneToAtoms", pickAtomPlaneInputMode);
@@ -354,10 +377,28 @@ void SliceModifierEditor::createUI(const RolloutInsertionParameters& rolloutPara
 	pickAtomPlaneInputModeActionProxy->setParent(this);
 	pickAtomPlaneInputModeActionProxy->setText(tr("Align plane to atoms"));
 	QWidget* alignPlaneToAtomsBtn = pickAtomPlaneInputModeActionProxy->requestWidget(rollout);
-	layout->addWidget(alignPlaneToAtomsBtn, 10, 0, 1, 3);
+	layout->addWidget(alignPlaneToAtomsBtn, 10, 0, 1, 2);
 
 	// Status label.
-	layout->addWidget(statusLabel(), 11, 0, 1, 3);
+	layout->addWidget(statusLabel(), 11, 0, 1, 2);
+}
+
+/******************************************************************************
+* Aligns the normal of the slicing plane with the X, Y, or Z axis.
+******************************************************************************/
+void SliceModifierEditor::onXYZNormal(const QString& link)
+{
+	SliceModifier* mod = static_object_cast<SliceModifier>(editObject());
+	if(!mod) return;
+
+	UNDO_MANAGER.beginCompoundOperation(tr("Set plane normal"));
+	if(link == "0")
+		mod->setNormal(Vector3(1,0,0));
+	else if(link == "1")
+		mod->setNormal(Vector3(0,1,0));
+	else if(link == "2")
+		mod->setNormal(Vector3(0,0,1));
+	UNDO_MANAGER.endCompoundOperation();
 }
 
 /******************************************************************************
