@@ -31,6 +31,7 @@
 #include <core/gui/properties/FilenamePropertyUI.h>
 #include <core/gui/properties/BooleanActionPropertyUI.h>
 #include <core/gui/properties/SubObjectParameterUI.h>
+#include <core/gui/dialogs/ImportFileDialog.h>
 #include <core/data/importexport/ImportExportManager.h>
 #include "AtomsImportObject.h"
 
@@ -157,6 +158,10 @@ bool AtomsImportObject::reloadInputFile()
 			TimeInterval interval(0, ANIM_MANAGER.frameToTime((parser()->numberOfMovieFrames()-1) * framesPerSnapshot()));
 			ANIM_MANAGER.setAnimationInterval(interval);
 		}
+		else {
+			ANIM_MANAGER.setAnimationInterval(TimeInterval(ANIM_MANAGER.frameToTime(0)));
+			ANIM_MANAGER.setTime(ANIM_MANAGER.frameToTime(0));
+		}
 
 		return true;
 	}
@@ -232,7 +237,7 @@ void AtomsImportObject::loadFromStreamComplete()
 			reloadInputFile();
 		}
 		catch(Exception& ex) {
-			ex.prependGeneralMessage(tr("Failed to restore atomistic data from external file. Sorry, your atoms are gone. Saved path to external data file: %1").arg(inputFile()));
+			ex.prependGeneralMessage(tr("Failed to restore atom data from external file. Sorry, your atoms are gone. Non-existing external data file: %1").arg(inputFile()));
 			ex.showError();
 		}
 	}
@@ -266,77 +271,27 @@ QString AtomsImportObject::schematicTitle()
 ******************************************************************************/
 void AtomsImportObject::showSelectionDialog(QWidget* parent)
 {
-	// Build filter string.
-	QStringList filterStrings;
-	Q_FOREACH(const ImportExportDescriptor& descr, IMPORTEXPORT_MANAGER.importers()) {
-		if(descr.pluginClass()->isKindOf(PLUGINCLASSINFO(AtomsFileParser))) {
-			filterStrings << QString("%1 (%2)").arg(descr.fileFilterDescription(), descr.fileFilter());
-		}
-	}
-	OVITO_ASSERT(!filterStrings.isEmpty());
-	filterStrings.prepend(tr("<Auto-detect file format> (*)"));
-
-	QSettings settings;
-
-	// Let the user select a file.
-	QFileDialog dialog(parent, tr("Import"));
-	dialog.setFilters(filterStrings);
-	dialog.setAcceptMode(QFileDialog::AcceptOpen);
-	dialog.setFileMode(QFileDialog::ExistingFile);
-
-	// Go to the last directory used.
-	QString lastImportDirectory = settings.value("file/import/last_import_dir").toString();
-	if(!lastImportDirectory.isEmpty())
-		dialog.setDirectory(lastImportDirectory);
-	// Select the last import filter being used ...
-	QString lastImportFilter = settings.value("file/import/last_import_filter").toString();
-	if(!lastImportFilter.isEmpty())
-		dialog.selectFilter(lastImportFilter);
-
-	if(!dialog.exec())
-		return;
-
-	QStringList filesToImport = dialog.selectedFiles();
-	if(filesToImport.isEmpty())
-		return;
-	QString importFile = filesToImport.front();
-
-	// Remember directory for the next time...
-	settings.setValue("file/import/last_import_dir", dialog.directory().absolutePath());
-	// Remember import filter for the next time...
-	settings.setValue("file/import/last_import_filter", dialog.selectedFilter());
-
-	ViewportSuspender noVPUpdate;
 	try {
+		// Let the user select a file.
+		ImportFileDialog dialog(parent, tr("Import"));
+		if(!dialog.exec())
+			return;
+
 		// Create a parser object based on the selected filename filter.
-		int importFilterIndex = filterStrings.indexOf(dialog.selectedFilter()) - 1;
-		AtomsFileParser::SmartPtr newParser;
-		if(importFilterIndex < 0) {
-			// Perform auto-detection of file format.
-			ImporterExporter::SmartPtr importer = IMPORTEXPORT_MANAGER.autodetectFileFormat(importFile);
-			if(!importer)
-				throw Exception(tr("Could not auto-detect file format. Please specify the format explicitly."));
-			newParser = dynamic_object_cast<AtomsFileParser>(importer);
-			if(!newParser)
-				throw Exception(tr("You did not select a file that contains atomistic data."));
-		}
-		else {
-			Q_FOREACH(const ImportExportDescriptor& descr, IMPORTEXPORT_MANAGER.importers()) {
-				if(descr.pluginClass()->isKindOf(PLUGINCLASSINFO(AtomsFileParser))) {
-					if(importFilterIndex-- == 0) {
-						newParser = static_object_cast<AtomsFileParser>(descr.createService());
-						break;
-					}
-				}
-			}
-			if(!newParser)
-				throw Exception(tr("Something went wront. Please report this error."));
-		}
+		QString importFile = dialog.fileToImport();
+		ImporterExporter::SmartPtr importer = dialog.createParser();
+		if(!importer) return;
+		AtomsFileParser::SmartPtr newParser = dynamic_object_cast<AtomsFileParser>(importer);
+		if(!newParser)
+			throw Exception(tr("You did not select a file that contains atomistic data."));
+
 		// Try to re-use the existing parser.
 		if(parser() && parser()->pluginClassDescriptor() == newParser->pluginClassDescriptor())
 			newParser = parser();
 		CHECK_OBJECT_POINTER(newParser.get());
 
+		ViewportSuspender noVPUpdate;
+
 		// Scan the input file.
 		if(!newParser->setInputFile(importFile))
 			return;
@@ -350,7 +305,6 @@ void AtomsImportObject::showSelectionDialog(QWidget* parent)
 	}
 	catch(const Exception& ex) {
 		ex.showError();
-		return;
 	}
 }
 
