@@ -249,7 +249,7 @@ DataRecordParserHelper::DataRecordParserHelper(const ColumnChannelMapping* mappi
 		throw Exception(tr("Cannot parse more than %1 data columns from the input file.").arg(MAXIMUM_ATOM_COLUMN_COUNT));
 
 	// Create data channels in the destination object.
-	for(int i=0; i<mapping->columnCount(); i++) {
+	for(int i = 0; i < mapping->columnCount(); i++) {
 		DataChannel* channel = NULL;
 		int channelType = mapping->getChannelType(i);
 		size_t vectorComponent = mapping->getVectorComponent(i);
@@ -323,8 +323,9 @@ void DataRecordParserHelper::storeAtom(int atomIndex, char* dataLine)
 		while(*dataLine != '\0' && (*dataLine == ' ' || *dataLine == '\t'))
 			++dataLine;
 		tokens[ntokens] = dataLine;
-		while(*dataLine != '\0' && *dataLine != ' ' && *dataLine != '\t')
+		while(*dataLine != '\0' && *dataLine != ' ' && *dataLine != '\t' && *dataLine != '\n' && *dataLine != '\r')
 			++dataLine;
+		if(*dataLine == '\n' || *dataLine == '\r') *dataLine = '\0';
 		if(dataLine != tokens[ntokens]) ntokens++;
 		if(*dataLine == '\0') break;
 		*dataLine = '\0';
@@ -358,7 +359,7 @@ void DataRecordParserHelper::storeAtom(int atomIndex, int ntokens, const char* t
 		atomIndex = strtoul(tokens[atomIndexColumn], &endptr, 10);
 		if(*endptr) throw Exception(tr("Invalid integer value in column %1 (atom index): \"%2\"").arg(atomIndexColumn+1).arg(tokens[atomIndexColumn]));
 		if(atomIndex > (int)destination->atomsCount())
-			throw Exception(tr("Atom index is out of range. This error is usually caused by partial data files, which contain only a subset of the total number of atoms in the system. To avoid this error, do NOT read in the atom IDs/indices during import. The erroneous atom index is %1, but number of atoms in input file is only %2.").arg(tokens[atomIndexColumn]).arg(destination->atomsCount()));
+			throw Exception(tr("Atom index is out of range. This error is usually caused by partial snapshot files which contain only a subset of the atoms in the simulation. To avoid this error, do NOT load the atom IDs/tag into the Atom Index data channel when importing the snapshot by clearing the corresponding field in the column mapping dialog. The erroneous atom index encountered in the input file is %1, but the total number of atoms in the file is only %2.").arg(tokens[atomIndexColumn]).arg(destination->atomsCount()));
 		else if(atomIndex < 1)
 			throw Exception(tr("Found non-positive atom index: %1.").arg(tokens[atomIndexColumn]));
 		atomIndex--;
@@ -397,6 +398,8 @@ void DataRecordParserHelper::storeAtom(int atomIndex, int ntokens, const char* t
 			else {
 				// Automatically create an atom type if a new type number is encountered.
 				if(!*endptr) {
+					if(d < 0)
+						throw Exception(tr("Negative atom type encountered in column %1. Atom type entries must be non-negative.").arg(columnIndex+1));
 					atypeChannel->setIntComponent(atomIndex, mapping->getVectorComponent(columnIndex), d);
 					atypeChannel->createAtomType(d);
 				}
@@ -436,7 +439,7 @@ void DataRecordParserHelper::storeAtom(int atomIndex, const double* values, int
 	if(atomIndexColumn >= 0 && atomIndexColumn < nvalues) {
 		atomIndex = (int)values[atomIndexColumn];
 		if(atomIndex > (int)destination->atomsCount())
-			throw Exception(tr("Atom index is out of range. This error is usually caused by partial data files, which contain only a subset of the total number of atoms in the system. To avoid this error, do NOT read in the atom IDs/indices during import. The erroneous atom index is %1, but number of atoms in input file is only %2.").arg(atomIndex).arg(destination->atomsCount()));
+			throw Exception(tr("Atom index is out of range. This error is usually caused by partial snapshot files which contain only a subset of the atoms in the simulation. To avoid this error, do NOT load the atom IDs/tag into the Atom Index data channel when importing the snapshot by clearing the corresponding field in the column mapping dialog. The erroneous atom index encountered in the input file is %1, but the total number of atoms in the file is only %2.").arg(atomIndex).arg(destination->atomsCount()));
 		else if(atomIndex < 1)
 			throw Exception(tr("Found non-positive atom index: %1.").arg(atomIndex));
 		atomIndex--;
@@ -470,8 +473,11 @@ void DataRecordParserHelper::storeAtom(int atomIndex, const double* values, int
 			d = (int)*value;
 			(*channel)->setIntComponent(atomIndex, mapping->getVectorComponent(columnIndex), d);
 			AtomTypeDataChannel* atypeChannel = dynamic_object_cast<AtomTypeDataChannel>(*channel);
-			if(atypeChannel)
+			if(atypeChannel) {
+				if(d < 0)
+					throw Exception(tr("Negative atom type encountered in column %1. Atom type entries must be non-negative.").arg(columnIndex+1));
 				atypeChannel->createAtomType(d);
+			}
 		}
 	}
 }
