@@ -58,7 +58,7 @@ DEFINE_FLAGS_VECTOR_REFERENCE_FIELD(ModifierStack, ObjectNode, "SelectedNodes",
 * Initializes the object.
 ******************************************************************************/
 ModifierStack::ModifierStack(ModifyCommandPage* modifyPage)
-	: RefMaker(), page(modifyPage), nextObjectToSelect(NULL), needStackUpdate(false), _aboutRollout(NULL)
+	: RefMaker(), page(modifyPage), nextObjectToSelect(NULL), needStackUpdate(false)
 {
 	INIT_PROPERTY_FIELD(ModifierStack, stackEntries);
 	INIT_PROPERTY_FIELD(ModifierStack, selectedNodes);
@@ -321,15 +321,11 @@ void ModifierStack::updatePropertiesPanel()
 		updateAvailableActions(NULL);
 
 		// Whenever no object is selected, show the about panel with information about the program.
-		if(_aboutRollout == NULL)
-			createAboutRollout();
-		if(_aboutRollout != NULL)
-			_aboutRollout->show();	
+		BrandingManager::primaryBranding()->showAboutPanel(page->propertiesPanel);
 	}
 	else {
 		// Hide about panel first.
-		if(_aboutRollout != NULL)
-			_aboutRollout->hide();	
+		BrandingManager::primaryBranding()->hideAboutPanel(page->propertiesPanel);
 
 		page->propertiesPanel->setEditObject(selectedEntry->commonObject());
 		updateAvailableModifiers(selectedEntry);
@@ -357,27 +353,6 @@ bool ModifierStack::modifierCategoryOrdering(const ModifierStack::ModifierCatego
     return QString::compare(a.label, b.label, Qt::CaseInsensitive) < 0;
 }
 
-/******************************************************************************
-* This method prepares the about panel with information about the program.
-******************************************************************************/
-void ModifierStack::createAboutRollout() 
-{
-	QLabel* label = new QLabel();
-	label->setTextInteractionFlags(Qt::TextBrowserInteraction);
-		
-	label->setOpenExternalLinks(true);
-	label->setTextFormat(Qt::RichText);
-	label->setWordWrap(true);
-	label->setMargin(6);
-
-	OVITO_ASSERT(!BrandingManager::brandings().empty());
-	Branding::SmartPtr branding = BrandingManager::brandings().back();
-	label->setText(branding->aboutPanelContent());
-
-	_aboutRollout = page->propertiesPanel->addRollout(label, branding->productName());
-}
-
-
 /******************************************************************************
 * Gathers all defined modifier categories from the plugin manifests.
 ******************************************************************************/
