@@ -78,11 +78,16 @@ ModifyCommandPage::ModifyCommandPage() : CommandPanelPage()
 		virtual QSize sizeHint() const { return QSize(256, 130); }
 	};
 
-	subLayout = new QHBoxLayout();
+	QSplitter* splitter = new QSplitter(Qt::Vertical);
+	splitter->setChildrenCollapsible(false);
+
+	QWidget* upperContainer = new QWidget();
+	splitter->addWidget(upperContainer);
+	subLayout = new QHBoxLayout(upperContainer);
 	subLayout->setContentsMargins(0,0,0,0);
 	subLayout->setSpacing(0);
 
-	stackBox = new ModifierStackListView(this);
+	stackBox = new ModifierStackListView(upperContainer);
 	stackBox->setModel(stack->listModel());
 	connect(stackBox->selectionModel(), SIGNAL(selectionChanged(const QItemSelection&, const QItemSelection&)), this, SLOT(onModifierStackSelectionChanged()));
 	connect(stackBox, SIGNAL(doubleClicked(const QModelIndex&)), this, SLOT(onModifierStackDoubleClicked(const QModelIndex&)));
@@ -91,8 +96,9 @@ ModifyCommandPage::ModifyCommandPage() : CommandPanelPage()
 
 	QToolBar* editToolbar = new QToolBar(this);
 	editToolbar->setOrientation(Qt::Vertical);
-	editToolbar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
+#ifndef Q_WS_MAC
 	editToolbar->setStyleSheet("QToolBar { padding: 0px; margin: 0px; border: 0px none black; }");
+#endif
 	subLayout->addWidget(editToolbar);
 
 	ActionProxy* deleteModifierAction = ACTION_MANAGER.addAction(new Action(ACTION_MODIFIER_DELETE), tr("Delete Modifier"), ":/core/command_panel/delete_modifier.png");
@@ -118,12 +124,14 @@ ModifyCommandPage::ModifyCommandPage() : CommandPanelPage()
 	connect(toggleModifierStateAction, SIGNAL(triggered(bool)), this, SLOT(onModifierToggleState(bool)));
 	editToolbar->addAction(toggleModifierStateAction);
 
-	layout->addLayout(subLayout);
+	layout->addWidget(splitter);
 	layout->addSpacing(4);
 
 	// Create the properties panel.
-	propertiesPanel = new PropertiesPanel(this);
-	layout->addWidget(propertiesPanel, 1);
+	propertiesPanel = new PropertiesPanel(NULL);
+	propertiesPanel->setFrameStyle(QFrame::NoFrame | QFrame::Plain);
+	splitter->addWidget(propertiesPanel);
+	splitter->setStretchFactor(1,1);
 
 	connect(&selectionSetListener, SIGNAL(notificationMessage(RefTargetMessage*)), this, SLOT(onSelectionSetMessage(RefTargetMessage*)));
 }
