@@ -193,7 +193,7 @@ void Viewport::updateProjectionMatrix()
 	_cameraView.aspectRatio = aspectRatio();
 
 	Box3 sceneExtentsPlusMargin = lastSceneExtent().centerScale(5.0);
-	sceneExtentsPlusMargin.addBox(Box3(ORIGIN, 4000));
+	sceneExtentsPlusMargin.addBox(Box3(ORIGIN, 200));
 
 	// Update projection and view matrices.
 	_cameraView = getViewDescription(ANIM_MANAGER.time(), _cameraView.aspectRatio, sceneExtentsPlusMargin);
@@ -292,7 +292,7 @@ void Viewport::mousePressEvent(QMouseEvent* event)
 	if(isEnabled() == false) return;
 	VIEWPORT_MANAGER.setActiveViewport(this);
 	container()->setFocus(Qt::MouseFocusReason);
-	if(event->button() == Qt::RightButton && contextMenuArea.contains(event->pos())) {
+	if(/*event->button() == Qt::RightButton && */contextMenuArea.contains(event->pos())) {
         showViewportMenu(event->pos());
         return;
 	}
@@ -424,7 +424,7 @@ Ray3 Viewport::viewportRay(const Point2& viewportPoint)
 	if(isPerspectiveProjection())
 		return Ray3(ORIGIN + inverseViewMatrix().getTranslation(), dir);
 	else
-		return Ray3(p2, dir);
+		return Ray3(p2 - dir * (currentView().zfar - currentView().znear), dir);
 }
 
 /******************************************************************************
@@ -583,7 +583,7 @@ void Viewport::zoomToExtents(SceneRenderer::SceneExtentsMode mode)
 }
 
 /******************************************************************************
-* Displays the context menu for thsi viewport.
+* Displays the context menu for this viewport.
 ******************************************************************************/
 void Viewport::showViewportMenu(const QPoint& pos)
 {
