@@ -48,10 +48,10 @@ public:
 	Color ambient;
 	Color diffuse;
 	Color specular;
+	Color selfIllum;
 	FloatType shininess;
 	FloatType shinStrength;
 	FloatType opacity;
-	FloatType selfIllum;
 
 	/// Default constructor.
 	Window3DMaterial() {
@@ -61,7 +61,7 @@ public:
 		shininess = 0.1f;
 		shinStrength = 0.1f;
 		opacity = 1.0;
-		selfIllum = 0.0;
+		selfIllum = Color(0, 0, 0);
 	}
 };
 
@@ -451,6 +451,12 @@ public:
 	/// \note This is an internal method and should not be called.
 	static void postWindowUpdates();
 
+	/// \brief Prepares the window for the rendering pass.
+	void beginFrame();
+
+	/// \brief Finishes the rendering pass.
+	void endFrame();
+
 	/// \brief Enlarges the bounding box that encompasses the whole rendered scene.
 	/// \param box The box to be added to the internal scene extent measure. This box must be given in world space.
 	///
@@ -537,12 +543,6 @@ protected:
 	/// \note This method must be implemented in derived classes.
 	virtual void renderWindow() = 0;
 
-	/// \brief Prepares the window for the rendering pass.
-	void beginFrame();
-
-	/// \brief Finishes the rendering pass.
-	void endFrame();
-
 	/// \brief Event handler for the widget's paint events.
 	virtual void paintEvent(QPaintEvent* event);
 
@@ -650,7 +650,7 @@ private:
 	/// Clips a triangle and performs a hit test.
 	void hitTestClippedTriangle(const Vector4 clipPoints[3]);
 
-    /// This logs a single hit that has been detected.
+    	/// This logs a single hit that has been detected.
 	void logHit(FloatType zvalue = 0) {
 		// Look only for the closest hit.
 		if(_closestHitDistance == HIT_TEST_NONE || zvalue < _closestHitDistance)
