@@ -137,7 +137,14 @@ bool CalculateIntrinsicStrainModifier::calculate(AtomsObject* atomsObject, Float
 	ProgressIndicator progress(tr("Calculating intrinsic strain tensors."), atomsObject->atomsCount(), suppressDialogs);
 
 	// Prepare the neighbor list.
-	OnTheFlyNeighborList neighborList(atomsObject, nearestNeighborList()->nearestNeighborCutoff());
+	OnTheFlyNeighborList neighborList(nearestNeighborList()->nearestNeighborCutoff());
+	if(!neighborList.prepare(atomsObject, suppressDialogs)) {
+		deformationGradientChannel->setSize(0);
+		strainTensorChannel->setSize(0);
+		hydrostaticStrainChannel->setSize(0);
+		shearStrainChannel->setSize(0);
+		return false;
+	}
 
 	// Prepare the output channels.
 	deformationGradientChannel->setSize(atomsObject->atomsCount());
