@@ -156,7 +156,11 @@ bool FindGrains::performAnalysis(AtomsObject* input, DataChannel* outputClusterC
 	progress.isCanceled();
 
 	// Prepare the neighbor list.
-	OnTheFlyNeighborList neighborList(input, nearestNeighborCutoff);
+	OnTheFlyNeighborList neighborList(nearestNeighborCutoff);
+	if(!neighborList.prepare(input, suppressDialogs)) {
+		outputClusterChannel->setSize(0);
+		return false;
+	}
 
 	// Put together a list of weighted edges in the nearest neighbor graph.
 	QVector<GraphEdge> bulkEdges;
